;**************************************************************
;
; Serial port bridge for the Altair 8800c front panel board
;
;    This code runs on the host 8080 and serves as a bridge
;    between the PIC processsor on the Altair 8800c front
;    panel and a standard 88-2SIO serial port in the system.
;
;    The bridge is used in "safe mode" to allow the PIC to
;    communicate with the user via a console connected to
;    the 2SIO port in order to update the "normal mode"
;    firmware. It is presently not used in the normal mode
;    firmware.
;
;    For use in the safe mode firmware, set the SAFEMOD equate
;    to one to include the hex loader code for loading firm-
;    ware updates. If ever used in normal mode software, set
;    SAFEMOD to zero to reduce the size of this code.
;
;    Author: Mike Douglas
; 
;    Revision: 1.0 (11/17/18),  Initial release 
;
;**************************************************************
SAFEMOD	equ	1		;1=safe mode, 0=normal mode

CODBASE	equ	0080h		;base address of the code
BUFBASE	equ	0200h		;hex file buffer address

SIOCSR	equ	10h		;2SIO control/status register
SIODR	equ	11h		;2SIO data register
RDRF	equ	1		;receive data available mask 
TDRE	equ	2		;transmit ready bit mask

FPIN	equ	0FFh		;input port from front panel
FPOUT	equ	0FFh		;output port to front panel

cHXLOAD	equ	0A5h		;command to load hex file
sIDLE	equ	0AAh		;status to show done/idle
sHEXERR	equ	0AEh		;status=hex file error

CTRLC	equ	03h		;ASCII control-c

	org	CODBASE		;program start address
;--------------------------------------------------------------
;  Initialize 2SIO console port, then loop sending data
;  provided by the front panel out the serial port, and 
;  giving any bytes that come in the serial port to the
;  front panel processor. If the HEXLOAD command is provided
;  by the panel, receive and store an Intel hex file into
;  binBuf.
;--------------------------------------------------------------
	di			;make sure interrupts are disabled
	lxi	SP,CODBASE	;init stack pointer

	mvi	a,03h		;reset 2SIO
	out	SIOCSR
	mvi	a,11h		;8N2, /16, RTS low, no ints
	out	SIOCSR

; Tell the front panel we're idle, then delay 100us to give the
;    front panel processor time to see the IDLE state and set
;    up new output data

idleOut	mvi	a,sIDLE		;set idle state output
	out	FPOUT

idleDly	mvi	a,13		;(7) 7+13*15 cycles = 101us

delay	dcr	a		;(5)
	jnz	delay		;(10)

; Look for a new byte from the front panel

loop	in	FPIN		;get byte from front panel
	mov	b,a		;   twice to ensure it is
	in	FPIN		;   not in the middle of
	cmp	b		;   being changed
	jnz	loop		;loop again if changing

 if SAFEMOD
	cpi	cHXLOAD		;hex file load requested?
	jz	hexLoad		;yes, go load hex file
 endif

	cpi	sIDLE		;data byte to send?
	jnz	sndByte		;yes, go send it

; Check for new byte in from serial port

	in	SIOCSR		;see if serial byte received
	rrc			;RDRF is in LS bit
	jnc	loop		;nothing, repeat loop

	in	SIODR		;get the new serial byte
	out	FPOUT		;give it to the front panel
	jmp	loop		;repeat

; Transmit the byte provided by the front panel out the serial port


sndByte	in	SIOCSR		;wait for xmit ready before exiting
	ani	TDRE
	jz	sndByte

	mov	a,b		;A=character to send
	out	SIODR		;transmit the byte provided
	jmp	idleOut		;tell front panel xmit is done and repeat

 if SAFEMOD
;--------------------------------------------------------------------
;  hexLoad - Load Intel hex file into binBuf. Addresses below
;     1000h are ignored (protected flash area where the safe
;     mode code and boot loader reside). Addresses above
;     valid flash memory are ignored as well. Incoming address
;     1000h maps to the first location in binBuf.
;
;     Addresses in the hex file are 2x the program word address
;     in the PIC processor. This program accepts PIC addresses
;     0800h-0FFFh which come in the hex file as 1000h-2000h.
;     (The protected safe mode code resides at 0000h-0800h in
;     the PIC.)
;
;     sIDLE is returned to the front panel by OUT 0FFh if the 
;     file is received with no errors and no memory errors are
;     detected. Otherwise, sHEXERR is returned.
;--------------------------------------------------------------------
hexload	xra	a		;init flag to show we're not skipping
	sta	skipDat		;  data records

	mvi	a,sIDLE		;default exit status is "good"
	sta	extStat

; init binBuf to words of 3FFFh (unprogrammed flash word)

	lxi	h,binBuf	;HL->binary output buffer
	lxi	b,3FFFH		;BC=unprogrammed PIC word
	mvi	d,((binBuf+1000h) shr 8) and 0FFh  ;MSB at end
	
initLp	mov	m,c		;write LSB of FFh
	inx	h
	mov	m,b		;write MSB of 3Fh
	inx	h
	
	mov	a,d		;HL reach end?
	cmp	h
	jnz	initLp		;no, continue

; rdLine - Loop processing one line at a time from the hex file

rdLine	call	getNib		;returns zero true if ':' read
	jnz	rdLine		;not ':' keep looking

	mov	d,a		;init checksum to zero
	mvi	a,'.'		;send '.' as progress rick
	out	SIODR
	
	call	getByte		;get byte count
	mov	a,e		;test byte for zero
	ora	a
	jz	flush		;count of 0 means end of data, exit
	
	mov	b,e		;B=byte count

	call	getByte		;MSB of address in H
	mov	h,e	
	call	getByte		;LSB of address in L
	mov	l,e
	
	call	getByte		;get record type
	mov	a,e
	cpi	4		;extended address record?
	jnz	dataRec		;no, assume its a data record
	
; extended address record - upper 16 bits are zero if in program space.
;    If not zero, ignore subsequent data records.

	call	getByte		;put A31-A24 in B
	mov	b,e
	
	call	getByte		;get A23-A16
	mov	a,e		;combine bytes to make a flag
	ora	b
	sta	skipDat		;will skip data records if non-zero
	
	jmp	rdLine		;go get next record
	
; data record - do not process if skipDat is true or the address is
;    less than than 1000h (lower addresses are protected flash)

dataRec	lda	skipDat		;skipping data records?
	ora	a
	jnz	rdLine		;yes, ignore the record
	
	mov	e,b		;preserve byte count in E		
	lxi	b,-1000h	;subtract 1000h offset from address
	dad	b
	jnc	rdLine		;below 0x1000, skip it
	
	lxi	b,binBuf	;offset HL into binBuf
	dad	b
	mov	b,e		;restore byte count into B

dataLp	call	getByte		;read a data byte (2 hex digits)
	mov	m,e		;move to buffer
	
	mov	a,m		;verify the write to memory
	cmp	e
	jnz	dataErr		;match failed
	
	inx	h		;move to next byte
	dcr	b
	jnz	dataLp		;repeat for all data bytes

	call	getByte		;read and add in checksum
	jz	rdLine		;valid checksum, do next line

dataErr	mvi	a,sHEXERR	;set exit status to "hex error"
	sta	extStat

	jmp	rdLine		;go get next line
	
; flush - throw away received characters for the next second so the
;   characters don't go to the front panel monitor. DE=0 on entry.

flush	in	SIODR		;(10) flush possible character
	dcx	d		;(5) decrement timeout
	mov	a,d		;(5)
	ora	e		;(4)
	jnz	flush		;(10) loop until zero
	
	lda	extStat		;give exit status to the front panel
abort	out	FPOUT
	
	jmp	idleDly		;jump back to main loop

;--------------------------------------------------------------
; getByte - read one ASCII hex bytes and convert to binary
;    in E. Update checksum in D. Conversion errors here will
;    typically generate a checksum error.
;--------------------------------------------------------------
getByte	call	getNib		;get ASCII hex nibble in binary
	add	a		;put in msn, zero lsn
	add	a
	add	a
	add	a
	mov	e,a		;save byte with MSN in E

; 2nd byte (LSN)

	call	getNib
	add	e		;combine msn and lsn
	mov	e,a		;save in E
	
	add	d		;update checksum in D
	mov	d,a
	ret

;---------------------------------------------------------------
; getNib - read one ASCII hex nibble and convert to binary in A.
;    If the character read is ':' zero status is true and A is
;    zero. Otherwise, zero status is false.
;---------------------------------------------------------------
getNib	in	SIOCSR		;wait for new character
	rrc
	jnc	getNib
	
	in	SIODR		;get the character
	cpi	CTRLC		;abort requested?
	jZ	abort		;yes (stack not reset, but ok)

	sui	':'		;start of record character?
	rz			;yes, return with zero
	
	adi	':'-'0'		;'0' to 0
	cpi	10		;0-9 ?
	rc			;yes, done (zero flag is false)
	
	sui	7		;convert A-F
	ret			;zero flag is false

;---------------------------------------------------------------
; Data Storage
;---------------------------------------------------------------
skipDat ds	1		;non-zero means skip data records
byteCnt	ds	1		;count of bytes on current line
extStat	ds	1		;exit status

	org	BUFBASE		;fixed buffer location
binBuf	equ	$		;hex file 1000h maps to here

 endif  ;if SAFEMD

	end
