;*********************************************************************************
;
; Altair 8800c Front Panel State Machine  -- NORMAL MODE --
;
;    This file is the "normal mode" version of the front panel control program
;    that is normally run when the computer powers up. This code is NOT in
;    protected flash and can be updated while the front panel is in safe mode.
;
;    Author: Mike Douglas
; 
;  Revision: 1.0 (12/06/18), Initial release 
; 	 1.1 (08/02/20), Update the inject code for the two Intel hex file
;   		loaders. The new code runs at the specified load address
;		instead of relocating to high memory.
;
;*********************************************************************************
;
;  Processor is a 16F1828, 16 MHz Internal Oscillator w/o PLL
;
	list p=16f1828,b=12,r=dec
	include <p16f1828.inc>

SW_VERSION	equ	o'011'	;version 1.1 in octal

; Port A equates		;(initial value) in parentheses

smCYCLEn	equ	5	;(1) low to cycle 8080 one cycle
smRESET	equ	4	;(1) high to assert PRESET to bus
smPOCn	equ	3	;(In) POC* (power on clear) signal from bus
smSERCLK	equ	2	;(0)serial transfer clock 
			;test data in, then positive pulse, LSB first
			;data out, NOP, positive pulse, LSB first
smSERIN	equ	1	;(In) data in from shift registers (front panel switches)
smINLATn	equ	0	;(1) pulse low to latch switches into shift registers

; Port B equates

smDI_LATFFn	equ	7	;(1) high to show data-in bus on LEDs, low shows FF latch
smOUTLAT	equ	6	;(0) pulse high to latch shift reg -> CPU inject latch
smUNPROTn	equ	5	;(1) low to assert UNPROT to bus
smSEROUT	equ	4	;(0) serial data out -> shift reg for CPU inject latch

; Port C equates

smMWRITE	equ	7	;(0) high to assert MWRITE to bus
smFFIN	equ	6	;(In) positive pulse (<500ns) when IN from 0FFh performed
			;   by 8080. Captured by CCP4
smDATAOE	equ	5	;(0) high to enable output of CPU inject latch
smRUN	equ	4	;(0) high to let the 8080 run, low to stop the 8080
smFFOUT	equ	3	;(In) positive pulse (<500ns) when OUT to 0FFh performed
			;   by 8080. Captured by CCP2
smPROTn	equ	2	;(1) low to assert PROT to bus
smSSn	equ	1	;(1) low to assert single-step to bus
smEXTCLR	equ	0	;(0) high to assert EXTCLR to bus

; On the front panel board, five 8-bit serial shift registers are used to read all switches 
;    and the value displayed on the data LEDs. Pulse the smINLATn low to parallel load all
;    five shift registers. Use the serial clock and data (smSERCLK, smSERIN) to read the
;    shift registers. Five bytes come in, LSBit to MSBit in the following order: (note that
;    all five shift registers do NOT have to be read if all data is not required)
;	
;    1) Data LED latch
;    2) A7-A0
;    3) RESET - AUX command switch group (see bit positions below)
;    4) A15-A8
;    5) STOP/RUN - DEPOSIT command switch group (see bit positions below)
;

swSTOP	equ	2	;STOP up		
swRUN	equ	3	;RUN down
swSSTEP	equ	0	;SINGLE STEP up
swSSTEPD	equ	1	;SINGLE STEP down
swEXAMINE	equ	5	;EXAMINE up
swEXAMNEXT	equ	4	;EXAMINE NEXT down
swDEPOSIT	equ	7	;DEPOSIT up
swDEPNEXT	equ	6	;DEPOSIT NEXT down

swRESET	equ	2	;RESET up
swEXTCLEAR	equ	3	;EXTernal CLEAR down
swPROTECT	equ	0	;memory PROTECT up
swUNPROTECT	equ	1	;memory UNPROTECT down
swAUXLEFTU	equ	5	;left AUX up
swAUXLEFTD	equ	4	;left AUX down
swAUXRIGHTU	equ	7	;right AUX up
swAUXRIGHTD	equ	6	;right AUX down

; Linkage equates between safe and normal mode  **DO NOT CHANGE**

SAFE_MODE	equ	0x0000	;address of "safe mode" code
NORMAL_MODE	equ	0x0800	;address of "normal mode" code
INJECT_PTRS	equ	0x0005	;table of pointers to injectable 8080 code
INJECT_PCNT	equ	36	;number of pointers in the table

; Flags variable equates

fRUNNING	equ	0	;1=CPU running
fRELOCATE	equ	1	;1=relocatable 8080 code block for depositBlk

; Parameter flags

pPWR_RESET	equ	7	;1=automatic reset upon power-on
pOUTFF_LAT	equ	6	;1=OUT FFh latch enabled
pCODE_INJCT	equ	5	;1=code snippet injection enabled

pRATE_MASK	equ	0x07	;mask to isolate auto-step rate

; Misc Equates

JMP_INST	equ	0xC3	;8080 JMP instruction
NOP_INST	equ	0x00	;8080 NOP instruction

INVALID_PTR	equ	0x3FFF	;invalid pointer value
RELO_MASK	equ	0x40	;0x40 bit in MSB of address if code is relocatable
RELO_BIT	equ	6	;bit number for relocatable flag

;************************************************************************
;
;  Data Definitions
;
;************************************************************************

; Page 0 data
	cblock	0x20	;start of bank 0 RAM
prevA15_A8			;previous value of switches A15 to A8
prevA7_A0			;previous value of switches A7-A0
prevLeft4			;previous value of left four switches
prevRight4			;previous value of right four switches

swCount			;switch processing loop counter
inChar			;byte read from serial bridge
hexByte			;byte formed by hexToBin
aStepCount			;auto single-step count down timer
aStepDelay			;auto step delay, LSB=25ms

; The following bytes are the RAM image of the parameters from EEPROM. 
;    When safe mode is used to update the normal mode firmware, the EEPROM
;    is cleared to zero. Therefore, parameters should be designed so that
;    a value of zero is always a valid, fail-safe mode of operation.

EEparams			;MSN=auto step rate hz, LSN=parameter flags
EEpwrAddr			;MSB of power-on auto jump address (0=disabled)
EEresAddr			;MSB of reset auto jump address (0=disabled)
EEparamEnd			;used to compute length of parameter area
	endc

EEPARAM_LEN	equ	EEparamEnd-EEparams

; Data common to all pages

	cblock	0x70	;common RAM all pages
swA15_A8			;value of switches A15 to A8
swA7_A0			;value of switches A7-A0
swLeft4			;value of left four switches
swRight4			;value of right four switches
flags			;flags byte
shiftReg			;input and output shift register
byteCount			;multi-purpose byte counter
bitCount			;multi-purpose bit counter
delayCount			;delay counter for delay subroutine
temp			;multi-purpose temp byte
	endc


;************************************************************************
;
;  Program Space
;
;************************************************************************
	org	NORMAL_MODE		;located in 2nd half of flash
	
	goto	normInit		;enter at initialization point
	
;------------------------------------------------------------------------
;  injectPtrs - Table of pointers to 8080 code segments that the user
;    can select on the front panel and have injected into 8080 RAM. This
;    table is also present in the safe mode code. If a valid pointer is
;    present here, it supersedes the corresponding table entry in the
;    safe mode code space. 
;
;    THE FIRST INJECT_PCNT ENTRIES IN THIS TABLE MUST CORRESPOND ONE-
;    FOR-ONE WITH THE THE TABLE ENTRIES IN THE SAFE MODE CODE.
;
;  THIS TABLE CANNOT CROSS A 256 BYTE BOUNDARY
;------------------------------------------------------------------------
injectPtrs	equ	$

; 000
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;BASIC 1.0
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 004
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;4K BASIC 3.2
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 010
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;8K BASIC 3.2
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 014	
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;4K BASIC 4.0
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 020
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;8K BASIC 4.0
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 024
	dw	low(INVALID_PTR),high(INVALID_PTR)     ;Extended BASIC 4.0
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 030
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;v3.2 cassette leader detector 
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 034
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;Port echo routines
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 040
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;Kill the Bit
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;Pong
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 044
	dw	low(injHex2SioA),high(injHex2SioA)+RELO_MASK	   ;Hex loader 2SIO port A
	dw	low(injHex2SioB),high(injHex2SioB)+RELO_MASK	   ;Hex loader 2SIO port B
	dw	low(injMemTest),high(injMemTest)	   ;memory test (2SIO console)

injPtrsCnt	equ	($-injectPtrs)/2

;----------------------------------------------------------------------------
; swJumpTbl - Jump table for command switch processing.
;    Switches are processed in a loop from LSB to MSB for the right group
;    of switches then the same for the left group of switches. swCount
;    decrements from 16 downto 1 for this sequence. As each switch bit is
;    checked, swCount-1 is used as an index into the table below to process
;    the corresponding switch.
;
; THIS TABLE CANNOT CROSS A 256 BYTE BOUNDARY
;----------------------------------------------------------------------------
swJumpTbl	goto	doDeposit
	goto	doDepNext	
	goto	doExamine
	goto	doExamNext
	goto	doRun
	goto	doStop
	goto	doAutoStep
	goto	doStep

	goto	doAuxRightU
	goto	doAuxRightD
	goto	doAuxLeftU
	goto	doAuxLeftD
	goto	doExtClear
	goto	doReset
	goto	doUnprotect
	goto	doProtect 

; normInit - Initialize current state of all switches, put the value of the
;    sense switches (A15-A8) into the CPU inject latch.

normInit	equ	$
	banksel	LATA		;access latch registers
	bcf	LATA,smRESET	;release reset
	
	bcf	LATA,smINLATn	;latch all switches into shift registers
	nop			;give time to settle
	bsf	LATA,smINLATn
	
	call	readLatch		;read 1st shift register (ignored)
	
	call	readLatch		;A7-A0 switches
	movwf	prevA7_A0
	
	call	readLatch		;right command switches
	movwf	prevRight4
	
	call	readLatch		;A15-A8 switches
	movwf	prevA15_A8

	call	readLatch		;left command switches
	movwf	prevLeft4

	movfw	prevA15_A8		;sense switches into CPU inject latch
	call	writeLatch
	
; Read EEPROM parameters, reset 8080 if requested, init variables

	call	readEE		;read EEPROM params into RAM

	btfss	EEparams,pPWR_RESET	;see if 8080 reset requested
	goto	noReset		;no

	banksel	LATA		;access latch registers
	bsf	LATA,smRESET	;assert reset
	movlw	250		;hold for 250us
	call	delayUs
	bcf	LATA,smRESET	;de-assert reset
	banksel	PORTA		;return to bank 0

noReset	call	getStepDly		;convert step rate to delay
	movwf	aStepDelay

 	clrf	flags		;clear all flags
	clrf	aStepCount		;auto-step not active

; Check power on auto-jump and run state

	movfw	EEpwrAddr		;see if power on auto-jump address present
	btfsc	STATUS,Z		;if non-zero, auto jump present
	goto	main		;no auto jump, enter main loop

	movlw	JMP_INST		;do "examine" of start address
	call	injectW
	clrw			;inject low byte of address (zero)
	call	injectW
	movfw	EEpwrAddr		;inject high byte of address
	call	injectW

	call	doRunEx		;start the 8080
				;fall into main loop
		
;****************************************************************************
;
; Main Loop
;     Every 25ms, read all switches and take the appropriate actions.
;
;****************************************************************************
main	btfss	PIR1,TMR2IF		;wait for 25ms flag from timer 2
	goto	main

	bcf	PIR1,TMR2IF		;clear the TMR2 interrupt flag
	call	rdSwitches		;load current state of all switches

chkSwitches	movfw	swLeft4		;check left 4 command switches
	xorwf	prevLeft4,F		;changed switches will be 1
	movfw	swRight4		;check right 4 command switches
	xorwf	prevRight4,F	;changed switches will be 1

	movlw	high(swJumpTbl)	;set MSB of jump table
	movwf	PCLATH

; Loop through all 16 command switches and call the appropriate process routine
;    for any switch that has changed state. Switches are processed from LSB to
;    MSB for the right group of switches then the same for the left group of
;    switches. swCount decrements from 16 downto 1 for this sequence. As each
;    bit is checked, swCount-1 is used as an index into swJumpTbl to call the
;    proper subroutine to process the switch.

	movlw	16		;checking 16 switch states
	movwf	swCount
	
swLoop	btfss	prevRight4,0	;did switch change?
	goto	noChange		;no
	
	movlw	low(swJumpTbl-1)	;compute LSB of jump address
	addwf	swCount,W
	callw			;call appropriate switch processor
	
noChange	lsrf	prevLeft4,F		;shift MSByte
	rrf	prevRight4,F	;shift LSByte
	decfsz	swCount,F
	goto	swLoop
	
; See if auto single-step is active. If so, decrement the timeout and
;    execute a single-step if timed out.

	movfw	aStepCount		;is the delay non-zero?
	btfsc	STATUS,Z
	goto	movePrev		;no, so auto-step not in progress
	
	decfsz	aStepCount,F	;decrement step timer
	goto	movePrev		;not timed out yet
	
	call	doAutoStep		;do another step

; Move current to previous switch state, put sense switches into CPU inject latch

movePrev	movfw	swLeft4		;move current to previous
	movwf	prevLeft4
	movfw	swRight4
	movwf	prevRight4

	movfw	swA15_A8		;sense switches into CPU inject latch
	call	writeLatch

	goto	main		;do it all over again
	
;----------------------------------------------------------------------------
; doStop - Stop switch raised or released. If raised, set smRUN low to
;    stop the 8080 in a wait state. Force display of the data-in bus on
;    the LEDs. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doStop	btfsc	swLeft4,swSTOP	;switch raised?
	return			;released, exit
	
	bcf	flags,fRUNNING	;8080 is stopped 
	banksel	LATA		;select latches bank
	bcf	LATC,smRUN		;clear smRUN to stop the 8080
	call	delay10		;delay 10us to ensure completion
	bsf	LATB,smDI_LATFFn	;show data bus on LEDs
	
	banksel	PORTA		;return to bank 0
	return
	
;----------------------------------------------------------------------------
; doRun - Run switch depressed or released. If depressed, set smRUN high
;    to let the 8080 run. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doRun	btfsc	swLeft4,swRUN	;switch depressed or released?
	return			;released, exit
	
doRunEx	bsf	flags,fRUNNING	;8080 is running
	banksel	LATA		;select latches bank
	bsf	LATC,smRUN		;set smRUN high
	call	delay10		;delay 10us to ensure completion

	banksel	PORTA		;return to bank 0
	return

;----------------------------------------------------------------------------
; doStep - Step switch raised or released. If raised, assert single-step
;    to the CPU board and then smCYCLE to let the 8080 execute one machine
;    cycle. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doStep	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	

	btfsc	swLeft4,swSSTEP	;switch raised or released?
	return			;released, exit
	
doStepEx	equ	$
	banksel	LATA		;select latches bank
	bcf	LATC,smSSn		;assert single-step to the CPU board
	bcf	LATA,smCYCLEn	;let 8080 execute one machine cycle
	nop			;assert for 750ns
	nop
	bsf	LATA,smCYCLEn	;8080 will stop after next PSYNC
	call	delay10		;delay 10us to ensure completion
	bsf	LATC,smSSn		;de-assert single-step
	
	banksel	PORTA		;return to bank 0
	return

;----------------------------------------------------------------------------
; doAutoStep - Step switch depressed or released. If depressed, start
;    the auto-step timeout from aStepDelay. If released, make sure the
;    timeout conter is cleared.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doAutoStep	btfsc	swLeft4,swSSTEPD	;switch depressed or released?
	goto	dasRelease		;released
	
	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the activation
	
	movfw	aStepDelay		;set the autostep delay
	movwf	aStepCount
	btfsc	STATUS,Z
	return			;ignore if delay is zero
		
	goto	doStepEx

dasRelease	clrf	aStepCount		;turn off autostep
	return

;----------------------------------------------------------------------------
; doExamine - Examine switch raised or released. If raised, inject the
;    three cycles of a JMP instrucion with the jump address specified 
;    on the front panel address switches. If AUX-Right is also raised,
;    then this examine is a request to "examine" the EEPROM parameters.
;    This is done by placing the parameters into 8080 RAM starting at
;    address one. The firmware version is placed in location zero. No
;    action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doExamine	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	
	
	btfsc	swLeft4,swEXAMINE	;switch raised or released?
	return			;released, exit

	btfss	swRight4,swAUXRIGHTU	;AUX-Right also raised?
	goto	deEeprom		;yes, examine EEPROM params

; Do normal examine operation

	movlw	JMP_INST		;inject JMP opcode to CPU
	call	injectW
	movfw	swA7_A0		;inject low byte of address
	call	injectW
	movfw	swA15_A8		;inject high byte of address
	call	injectW

	return

; deEeprom - examine EEPROM parameters by putting them into RAM starting
;   at address one. The firmware version is placed at address zero.

deEeprom	call	examine0		;examine address zero

	movlw	SW_VERSION		;deposit software version
	call	depositInc		;deposit and increment address

	movfw	EEparams		;deposit 1st parameter byte
	call	depositInc

	movfw	EEpwrAddr		;deposit 2nd parameter byte
	call	depositInc

	movfw	EEresAddr		;deposit 3rd parameter byte
	call	doDepositW	
				;fall into examine address zero
; examine0 - examine address zero

examine0	movlw	JMP_INST		;inject JMP opcode to CPU
	call	injectW
	clrw			;inject low byte of address (zero)
	call	injectW
	clrw			;inject high byte of address (zero)
	call	injectW
	return

; depositInc - deposit W in the current location, then increment to next location

depositInc	call	doDepositW
	movlw	NOP_INST
	call	injectW

	return

;----------------------------------------------------------------------------
; doExamNext- Examine Next switch depressed or released. If depressed,
;    inject a NOP instruction to cause the CPU to go to the next address.
;    No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doExamNext	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	

	btfsc	swLeft4,swEXAMNEXT	;switch depressed or released?
	return			;released, exit
	
	movlw	NOP_INST		;inject NOP instruction to CPU
	call	injectW
	
	return

;----------------------------------------------------------------------------
; doDeposit - Deposit switch raised or released. If raised, deposit the
;     the value on switches A7-A0 into the current address by asserting
;     MWRITE. If AUX-Right is also raised, then this deposit is a request
;     to write the parameter bytes from 8080 RAM (starting at location one)
;     into EEPROM. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doDeposit	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	

	btfsc	swLeft4,swDEPOSIT	;switch raised or released?
	return			;released, exit

	btfss	swRight4,swAUXRIGHTU	;AUX-Right also raised?
	goto	ddEeprom		;yes, write EEPROM params
	
; Do normal deposit operation

doDepositEx	movfw	swA7_A0		;W=value to deposit

doDepositW	call	writeLatch		;write W to inject latch
	banksel	LATA		;select latches bank
	bsf	LATC,smDATAOE	;enable output of inject latch
	nop			;let data settle
	
	bsf	LATC,smMWRITE	;assert MWRITE to write data to RAM.
	movlw	8		;assert for about 8us like original Altair
	call	delayUs
	bcf	LATC,smMWRITE	;remove MWRITE

	nop			;give some hold time
	bcf	LATC,smDATAOE	;disable output of CPU inject latch

	banksel	PORTA		;return to bank 0
	return

; ddEeprom - write parameters starting from memory location 1 to EEPROM.

ddEeprom	call	examine0		;examine address zero

	call	read8080		;read EEparams byte
	movwf	EEparams		;save new byte in RAM

	call	read8080		;read EEpwrAddr byte
	movwf	EEpwrAddr		;save new byte in RAM

	call	read8080		;read EEresAddr byte
	movwf	EEresAddr		;save new byte in RAM
	
	call	getStepDly		;convert step rate to delay
	movwf	aStepDelay

	call	writeEE		;write the parameters to EEPROM
	call	examine0		;examine address zero on exit
	return

; read8080 - increment the 8080 memory location then return the value
;    shown on the data LEDs at that address in W.

read8080	movlw	NOP_INST		;NOP to increment 8080 address
	call	injectW

	banksel	LATA		;access latch registers
	bcf	LATA,smINLATn	;latch data LEDs into shift register
	nop			;time to settle over ribbon cable
	bsf	LATA,smINLATn

	call	readLatch		;read the 8080 data byte
	return

;----------------------------------------------------------------------------
; doDepNext- Deposit Next switch depressed or released. If depressed,
;    inject a NOP instruction to cause the CPU to go to the next address,
;    then jump to doDeposit to write the value on switches A7-A0 to
;    memory. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doDepNext	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	

	btfsc	swLeft4,swDEPNEXT	;switch depressed or released?
	return			;released, exit
	
	movlw	NOP_INST		;inject NOP instruction to CPU
	call	injectW
	movlw	250		;delay 500us after 8080 NOP
	call	delayUs		;  to duplicate original Altair
	movlw	250		;  timing. This causes a slight
	call	delayUs		;  LED flicker during deposit next
	
	goto	doDepositEx		;deposit value on switches
	
;----------------------------------------------------------------------------
; doReset - Reset switch raised or released. If raised, assert smRESET.
;    If released, see if AUX-RIGHT is raised. If so, this is a request
;    to restart in safe mode. Otherwise, de-assert smRESET and process
;    the reset auto-jump in EEresAddr if not zero.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
doReset	equ	$
	banksel	LATA		;access latch registers

	btfss	swRight4,swRESET	;switch raised or released?
	goto	drAssert		;raised, go assert reset
	bcf	LATA,smRESET	;released, de-assert reset
	
	banksel	PORTA		;return to bank 0
	btfsc	swRight4,swAUXRIGHTU	;AUX-Right also raised?
	goto	drChkJump		;no, go check for auto-jump

	clrf	PCLATH		;jump to safe mode at address zero
	goto	SAFE_MODE	

drChkJump	movfw	EEresAddr		;see if reset auto-jump address present
	btfsc	STATUS,Z		;if not zero, auto jump present
	return			;otherwise, just exit

	banksel	LATA		;select latches bank
	bcf	LATC,smRUN		;clear smRUN to stop the 8080
	call	delay10		;delay 10us to ensure completion

	movlw	JMP_INST		;do "examine" of start address
	call	injectW
	clrw			;inject low byte of address (zero)
	call	injectW
	movfw	EEresAddr		;inject high byte of address
	call	injectW
	
	btfsc	flags,fRUNNING	;were we running?
	call	doRunEx		;yes, start processor again
	return			

drAssert	bsf	LATA,smRESET	;assert reset
	banksel	PORTA		;return to bank 0
	return	
		
;----------------------------------------------------------------------------
; doExtClear - External Clear switch depressed or released. If depressed,
;    assert smEXTCLR. If released, de-assert smEXTCLR.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
doExtClear	equ	$
	banksel	LATA		;access latch registers

	btfss	swRight4,swEXTCLEAR	;switch depressed?	
	bsf	LATC,smEXTCLR	;yes, assert external clear

	btfsc	swRight4,swEXTCLEAR	;switch released?
	bcf	LATC,smEXTCLR	;yes, de-assert external clear

	banksel	PORTA		;return to bank 0
	return

;----------------------------------------------------------------------------
; doProtect - Protect switch raised or released. If raised and the CPU is
;    not running, then assert smPROT. If released, always de-assert 
;    smPROT just to be safe.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doProtect	btfsc	swRight4,swPROTECT	;switch raised or released?
	goto	dpRelease		;released

	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, exit

	banksel	LATA		;access latch registers
	bcf	LATC,smPROTn	;assert protect signal
	
	banksel	PORTA		;return to bank 0
	return

dpRelease	equ	$
	banksel	LATA		;access latch registers
	bsf	LATC,smPROTn	;turn off protect signal
	
	banksel	PORTA		;return to bank 0
	return

;----------------------------------------------------------------------------
; doUnprotect - Unprotect switch depressed or released. If depressed
;    and the CPU is not running, then assert smUNPROT. If released,
;    always de-assert smUNPROT just to be safe.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doUnprotect	btfsc	swRight4,swUNPROTECT	;switch depressed or released?
	goto	duRelease		;released

	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, exit

	banksel	LATA		;access latch registers
	bcf	LATB,smUNPROTn	;assert unprotect signal
	
	banksel	PORTA		;return to bank 0
	return

duRelease	equ	$
	banksel	LATA		;access latch registers
	bsf	LATB,smUNPROTn	;turn off unprotect signal
	
	banksel	PORTA		;return to bank 0
	return
	
;----------------------------------------------------------------------------
; doAuxLeftU - Left Aux switch raised or released. Do nothing.
;
; BankSel on exit: no change
;----------------------------------------------------------------------------
doAuxLeftU	return			;else, just exit


;----------------------------------------------------------------------------
; doAuxLeftD - Left Aux switch depressed or released. If depressed and
;    code injection is enabled, perform the code injection
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doAuxLeftD	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, exit
	
	btfsc	swRight4,swAUXLEFTD	;switch depressed or released?
	return			;released, just exit
	
	btfsc	EEparams,pCODE_INJCT	;code injection enabled?
	call	codeInject		;yes, perform code inject

	return

;----------------------------------------------------------------------------
; doAuxRightU - Right Aux switch raised or released. If raised, set
;    smDI_LATFFn high to display the data input bus on the data LEDs.
;
;    No action on release.
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
doAuxRightU	equ	$
	banksel	LATA		;access latch registers
	btfss	swRight4,swAUXRIGHTU	;switch raised?
	bsf	LATB,smDI_LATFFn	;yes, display the data in bus

	banksel	PORTA		;return to bank 0
	return

;----------------------------------------------------------------------------
; doAuxRightD - Right Aux switch depressed or released. If depressed and
;    the OUT 0FFh feature is enabled, set smDI_LATFFn low to display the
;    OUT 0FFh latch on the data LEDs. On release and the CPU is stopped,
;    then restore the data-in bus display on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doAuxRightD	btfss	EEparams,pOUTFF_LAT	;OUT 0FFh latch enabled?
	return			;no

	btfsc	swRight4,swAUXRIGHTD	;switch raised or released?
	goto	dardRelease		;released

	banksel	LATA		;access latch registers
	bcf	LATB,smDI_LATFFn	;display latch content on LEDs
	
	banksel	PORTA		;return to bank 0
	return	
	
dardRelease	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, exit

	banksel	LATA		;access latch registers
	bsf	LATB,smDI_LATFFn	;display data in bus
	
	banksel	PORTA		;return to bank 0
	return	

;----------------------------------------------------------------------------
; rdSwitches - Read the current state of all switches from the shift 
;   registers into the corresponding switch variables. Takes 121.75us
;   (487 cycles) including call and return.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
rdSwitches	equ	$
	banksel	LATA		;access latch registers

	bcf	LATA,smINLATn	;latch all switches into shift registers
	nop			;time to settle over ribbon cable
	bsf	LATA,smINLATn

	call	readLatch		;read 1st shift register (ignored)

	call	readLatch		;A7-A0 switches
	movwf	swA7_A0

	call	readLatch		;right action switches
	movwf	swRight4

	call	readLatch		;A15-A8 switches
	movwf	swA15_A8

	call	readLatch		;left action switches
	movwf	swLeft4

	return

;----------------------------------------------------------------------------
; injectW - Put the value passed in W into the CPU inject latch and
;    then assert smCYCLEn to let the 8080 run a machine cycle.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
injectW	call	writeLatch		;write W to CPU inject latch

	banksel	LATA		;access latch registers
	bsf	LATC,smDATAOE	;enable output of CPU inject latch
	
	bcf	LATA,smCYCLEn	;let 8080 execute one machine cycle
	nop			;assert for 750ns
	nop
	bsf	LATA,smCYCLEn	;8080 will stop after next PSYNC
	call	delay10		;give some time to finish

	bcf	LATC,smDATAOE	;disable output of CPU inject latch

	banksel	PORTA		;exit with bank 0 selected
	return
	
;----------------------------------------------------------------------------
; readLatch - Read the next 8 bits from the 40-bit shift register latch
;    and return in W. Data is read LSBit first. Takes 23.75us (95 cycles)
;    including call and return.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
readLatch	movlw	8		;reading 8 bits
	movwf	bitCount

rlLoop	lsrf	shiftReg,F		;shift towards LSB, bit 7 is 0
	banksel	PORTA		;access ports for input
	btfsc	PORTA,smSERIN	;test data bit
	bsf	shiftReg,7		;it was one

	banksel	LATA		;access latch registers
	bsf	LATA,smSERCLK	;clock the shift registers
	nop			;time to settle over ribbon cable
	bcf	LATA,smSERCLK

	decfsz	bitCount,F		;repeat for 8 bits
	goto	rlLoop

	movfw	shiftReg		;return result in W	
	banksel	PORTA		;return to bank 0
	return
	
;----------------------------------------------------------------------------
; writeLatch - Write the byte from W to the CPU inject latch. Data is
;    is first written to a shift register and then latched into an 8-bit
;    latch. Data is shifted out LSBit first. Takes 26.5us (106 cycles)
;    including call and return.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
writeLatch	movwf	shiftReg		;save value to write
	movlw	8		;writing 8 bits
	movwf	bitCount
	banksel	LATA		;access latch registers

wlLoop	btfss	shiftReg,0		;what is next data bit?
	goto	wlData0		;bit is zero
	
	bsf	LATB,smSEROUT	;bit is one
	goto	wlClock		;go clock in the bit
	
wlData0	bcf	LATB,smSEROUT	;bit is zero
	nop			;data setup time

wlClock	bsf	LATA,smSERCLK	;clock the shift register
	nop			;clock is also on ribbon cable
	bcf	LATA,smSERCLK

	lsrf	shiftReg,F		;move to next bit	
	decfsz	bitCount,F		;repeat for 8 bits
	goto	wlLoop
	
	bsf	LATB,smOUTLAT	;latch shift register to parallel latch
	bcf	LATB,smOUTLAT
	
	banksel	PORTA		;exit with bank 0 selected
	return

;----------------------------------------------------------------------------
; delay10 - Delay 10us
; delayUs - Delay W us including movlw n, call, and return. Minimum
;     delay is 3us.
;
; BankSel on exit: no change
;----------------------------------------------------------------------------
delay10	movlw	10		;delay 10us

delayUs	addlw	-2		;account for fixed overhead
	movwf	delayCount		;save delay count
	nop
	
duLoop	nop
	decfsz	delayCount,F
	goto	duLoop
	
	nop
	return

;----------------------------------------------------------------------------
; codeInject - Index into the injectPtrs table based on the operator
;   selected value on switches A7-A0. This selects a code segment to inject
;   into 8080 RAM. The local inject table is checked first. If the specified
;   entry is blank, then the inject table in the safe mode address space is
;   checked. If a valid pointer is found, control passes to depositBlk to
;   load the selected code into 8080 RAM. Otherwise, this routine simply
;   returns to the caller.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
; See if A7-A0 indexes to a valid pointer in injectPtrs, the pointer
;    table in this normal-mode code. If so, load the code it points
;    to into the 8080.
  
codeInject	movlw	injPtrsCnt		;switches A7-A0 < num pointers?
	subwf	swA7_A0,W
	btfsc	STATUS,C
	goto	ciChkSafe		;not in local table, try safe mode table

	lslf	swA7_A0,W		;W=2*lower 8 switches
	addlw	low(injectPtrs)	;FSR1->injectPtrs[swA7_A0]
	movwf	FSR1L
	movlw	high(injectPtrs)+0x80		
	movwf	FSR1H
	
	moviw	FSR1++		;FSR0->selected code snippet
	movwf	FSR0L
	moviw	FSR1++
	movwf	FSR0H

	sublw	high(INVALID_PTR)	;valid entry present?
	btfsc	STATUS,Z	
	goto	ciChkSafe		;no, go check safe mode table

	btfsc	FSR0H,RELO_BIT	;relocatable block bit set?
	bsf	flags,fRELOCATE	;yes, set relocatable flag
	movlw	~RELO_MASK		;clear the bit
	andwf	FSR0H,F
	
	goto	ciDeposit		;go deposit the block	

; ciChkSafe - See if A7-A0 indexes to a valid ponter in INJECT_PTRS, the pointer
;    table in the safe_mode block of code. If so, load the code it point to
;    into the 8080.

ciChkSafe	movlw	INJECT_PCNT		;switches A7-A0 < num pointers?
	subwf	swA7_A0,W
	btfsc	STATUS,C
	return			;beyond table size
	
	lslf	swA7_A0,W		;W=2*lower 8 switches
	addlw	low(INJECT_PTRS)	;FSR1->INJECT_PTRS[swA7_A0]
	movwf	FSR1L
	movlw	high(INJECT_PTRS)+0x80		
	movwf	FSR1H
	
	moviw	FSR1++		;FSR0->selected code snippet
	movwf	FSR0L
	moviw	FSR1++
	movwf	FSR0H
	
	sublw	high(INVALID_PTR)	;valid entry present?
	btfsc	STATUS,Z
	return			;invalid selection, exit

	btfsc	FSR0H,RELO_BIT	;relocatable block bit set?
	bsf	flags,fRELOCATE	;yes, set relocatable flag
	movlw	~RELO_MASK		;clear the bit
	andwf	FSR0H,F

ciDeposit	call	depositBlk		;deposit the block
	bcf	flags,fRELOCATE	;turn off possible relocate flag
	return

;----------------------------------------------------------------------------
; depositBlk - Deposit a block of data pointed to by FSR0 into 8080
;   memory. On exit, an "examine" of the specified start address is done
;   to set the 8080 PC to the start address. A data block is organized
;   as follows:
;
;       16 bit load address (LSB, MSB)
;       8 bit length (256 byte maximum, 0=256 bytes)
;       1-256 bytes of 8080 code
;       16 bit start address (LSB, MSB)
;
;   If the fRELOCATE flag is set, then the MSB of the load and start
;   address comes from switches A15-A8 instead of the value in the code
;   block entry. 
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
depositBlk	call	examFSR0		;do "examine" of load address

	moviw	FSR0++		;w=number of bytes to move
	movwf	byteCount

dbLoop	moviw	FSR0++		;w=next byte of 8080 program
	call	writeLatch		;put the byte in the inject latch
	
	banksel	LATA		;select latches bank
	bsf	LATC,smDATAOE	;enable output of inject latch
	nop			;let data settle
	
	bsf	LATC,smMWRITE	;assert MWRITE to write data to RAM.
	movlw	8		;assert for about 8us like original Altair
	call	delayUs
	bcf	LATC,smMWRITE	;remove MWRITE
	
	movlw	NOP_INST		;execute NOP to increment 8080 memory location
	call	injectW
	
	decfsz	byteCount,F		;repeat for all bytes in block
	goto	dbLoop

; Block of data has been loaded into 8080 RAM. Examine the specified start
;    address in case the caller wants to execute the code.

examFSR0	movlw	JMP_INST		;do "examine" of start address
	call	injectW

	moviw	FSR0++		;W=lsb of address
	call	injectW
	
	moviw	FSR0++		;msb of address
	btfsc	flags,fRELOCATE	;relocatable code block?
	movfw	swA15_A8		;yes, msb of address from switches A15-A8
	call	injectW
	return

;----------------------------------------------------------------------------
; readEE - Read the EExxx bytes from EEPROM into RAM page 1.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
readEE	movlw	EEparams		;point FSR0 to EEPROM RAM image
	movwf	FSR0L
	clrf	FSR0H

	banksel	EEADRL
	clrf	EEADRL		;start at address zero in EEPROM	
	bcf	EECON1,EEPGD	;select EEPROM, not flash
	bcf	EECON1,CFGS		;not configuration words

	movlw	EEPARAM_LEN		;number of bytes to read
	movwf	byteCount

reLoop	bsf	EECON1,RD		;read byte from EEPROM to EEDATL
	movfw	EEDATL		;w=byte read
	movwi	FSR0++		;store in RAM

	incf	EEADRL,F		;increment EEPROM address
	decfsz	byteCount,F		;repeat for all bytes
	goto	reLoop

	banksel	PORTA		;restore bank 0
	return

;----------------------------------------------------------------------------
; writeEE - Write the EExxx bytes from page 1 to EEPROM starting at 
;   address zero.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
writeEE	movfw	EEpwrAddr		;if power-on jump address present,
	btfss	STATUS,Z		
	bsf	EEparams,pPWR_RESET	;force power-on reset

	movlw	EEparams		;point FSR0 to EEPROM RAM image
	movwf	FSR0L
	clrf	FSR0H

	banksel	EEADRL
	clrf	EEADRL		;start at address zero in EEPROM	
	bcf	EECON1,EEPGD	;select EEPROM, not flash
	bcf	EECON1,CFGS		;not configuration words
	bsf	EECON1,WREN		;enable writes

	movlw	EEPARAM_LEN		;number of bytes to write
	movwf	byteCount

weLoop	moviw	FSR0++		;get byte to write
	movwf	EEDATL		;store for EEPROM write
	
	movlw	0x55		;write enable sequence...
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR		;set WR to start the write
	
weWait	btfsc	EECON1,WR		;wait for WR to clear
	goto	weWait
	
	incf	EEADRL,F		;increment EEPROM address
	decfsz	byteCount,F		;repeat for all bytes
	goto	weLoop
	
	bcf	EECON1,WREN		;EEPROM erased, disable writes
	banksel	PORTA		;restore bank 0
	return

;----------------------------------------------------------------------------
; getStepDly - Get the step rate (in hz) from the MSN of params and
;    return the equivalent delay between steps in W with an LS bit 
;    of 25ms.
;
; BankSel on exit: no change
;---------------------------------------------------------------------------
getStepDly	movfw	EEparams		;step rate is in EEparams
	andlw	pRATE_MASK		;get rate alone
	brw			;index into table by rate

	retlw	0		;no auto step
	retlw	40		;1hz (40*25ms = 1 second)
	retlw	20		;2hz
	retlw	13		;3hz
	retlw	10		;4hz
	retlw	8		;5hz	
	retlw	7		;6hz
	retlw	6		;7hz

;----------------------------------------------------------------------------
; Intel hex file loader, RELOCATABLE, first port on 2SIO
;----------------------------------------------------------------------------
adrHex2SioA	equ	0x0000		;load address in 8080

injHex2SioA	dw	low(adrHex2SioA)	;8080 load address
	dw	high(adrHex2SioA)
	dw	endHex2SioA-strHex2SioA	;length of block

strHex2SioA	equ	$

 dw 0x31,0x80,0x00,0x21,0xE1,0xE9,0xE5,0xCD,0x7E,0x00,0x2E,0x13,0xE5,0xE1,0x2E,0x23
 dw 0x54,0x1E,0x1D,0x3B,0x3B,0x7E,0xFE,0xFD,0xD5,0xC0,0xD1,0x7C,0x77,0x2C,0xC0,0x2E
 dw 0x23,0xF9,0xE9,0x3E,0x03,0xD3,0x10,0x3E,0x15,0xD3,0x10,0xCD,0xCF,0xFD,0x0D,0x0A
 dw 0x53,0x65,0x6E,0x64,0x20,0x46,0x69,0x6C,0x65,0x20,0x4E,0x6F,0xF7,0x0E,0x00,0xCD
 dw 0xCF,0xFD,0x0D,0x8A,0xCD,0xBC,0xFD,0xD6,0x3A,0xC2,0x44,0xFD,0x57,0xCD,0x8F,0xFD
 dw 0x7B,0xB7,0xCA,0x81,0xFD,0x43,0x0C,0xCD,0x8F,0xFD,0x63,0xCD,0x8F,0xFD,0x6B,0x0D
 dw 0xCD,0x8F,0xFD,0xCD,0x8F,0xFD,0x73,0x23,0x05,0xC2,0x63,0xFD,0xCD,0x8F,0xFD,0xCA
 dw 0x3F,0xFD,0xCD,0xCF,0xFD,0x0D,0x0A,0x45,0x72,0x72,0x6F,0x72,0x0D,0x8A,0xC3,0x8A
 dw 0xFD,0xCD,0xCF,0xFD,0x44,0x6F,0x6E,0x65,0x0D,0x8A,0xDB,0x11,0xC3,0x8A,0xFD,0xCD
 dw 0xBC,0xFD,0xCD,0xAB,0xFD,0xDA,0x72,0xFD,0x87,0x87,0x87,0x87,0x5F,0xCD,0xBC,0xFD
 dw 0xCD,0xAB,0xFD,0xDA,0x72,0xFD,0x83,0x5F,0x82,0x57,0xC9,0xD6,0x30,0xD8,0xFE,0x0A
 dw 0xDA,0xBA,0xFD,0xD6,0x11,0xD8,0xC6,0x0A,0xFE,0x10,0x3F,0xC9,0xDB,0x10,0x0F,0xD2
 dw 0xBC,0xFD,0x79,0xB7,0xCA,0xCC,0xFD,0xDB,0x11,0xD3,0x11,0xC9,0xDB,0x11,0xC9,0xE1
 dw 0xDB,0x10,0xE6,0x02,0xCA,0xD0,0xFD,0x7E,0xE6,0x7F,0xD3,0x11,0xB6,0x23,0xF2,0xD0
 dw 0xFD,0xE9
	
endHex2SioA	dw	low(adrHex2SioA)	;8080 start address
	dw	high(adrHex2SioA)

;----------------------------------------------------------------------------
; Intel hex file loader, RELOCATABLE, 2nd port on 2SIO
;----------------------------------------------------------------------------
adrHex2SioB	equ	0x0000		;load address in 8080

injHex2SioB	dw	low(adrHex2SioB)	;8080 load address
	dw	high(adrHex2SioB)
	dw	endHex2SioB-strHex2SioB	;length of block

strHex2SioB	equ	$

 dw 0x31,0x80,0x00,0x21,0xE1,0xE9,0xE5,0xCD,0x7E,0x00,0x2E,0x13,0xE5,0xE1,0x2E,0x23
 dw 0x54,0x1E,0x1D,0x3B,0x3B,0x7E,0xFE,0xFD,0xD5,0xC0,0xD1,0x7C,0x77,0x2C,0xC0,0x2E
 dw 0x23,0xF9,0xE9,0x3E,0x03,0xD3,0x12,0x3E,0x15,0xD3,0x12,0xCD,0xCF,0xFD,0x0D,0x0A
 dw 0x53,0x65,0x6E,0x64,0x20,0x46,0x69,0x6C,0x65,0x20,0x4E,0x6F,0xF7,0x0E,0x00,0xCD
 dw 0xCF,0xFD,0x0D,0x8A,0xCD,0xBC,0xFD,0xD6,0x3A,0xC2,0x44,0xFD,0x57,0xCD,0x8F,0xFD
 dw 0x7B,0xB7,0xCA,0x81,0xFD,0x43,0x0C,0xCD,0x8F,0xFD,0x63,0xCD,0x8F,0xFD,0x6B,0x0D
 dw 0xCD,0x8F,0xFD,0xCD,0x8F,0xFD,0x73,0x23,0x05,0xC2,0x63,0xFD,0xCD,0x8F,0xFD,0xCA
 dw 0x3F,0xFD,0xCD,0xCF,0xFD,0x0D,0x0A,0x45,0x72,0x72,0x6F,0x72,0x0D,0x8A,0xC3,0x8A
 dw 0xFD,0xCD,0xCF,0xFD,0x44,0x6F,0x6E,0x65,0x0D,0x8A,0xDB,0x13,0xC3,0x8A,0xFD,0xCD
 dw 0xBC,0xFD,0xCD,0xAB,0xFD,0xDA,0x72,0xFD,0x87,0x87,0x87,0x87,0x5F,0xCD,0xBC,0xFD
 dw 0xCD,0xAB,0xFD,0xDA,0x72,0xFD,0x83,0x5F,0x82,0x57,0xC9,0xD6,0x30,0xD8,0xFE,0x0A
 dw 0xDA,0xBA,0xFD,0xD6,0x11,0xD8,0xC6,0x0A,0xFE,0x10,0x3F,0xC9,0xDB,0x12,0x0F,0xD2
 dw 0xBC,0xFD,0x79,0xB7,0xCA,0xCC,0xFD,0xDB,0x13,0xD3,0x13,0xC9,0xDB,0x13,0xC9,0xE1
 dw 0xDB,0x12,0xE6,0x02,0xCA,0xD0,0xFD,0x7E,0xE6,0x7F,0xD3,0x13,0xB6,0x23,0xF2,0xD0
 dw 0xFD,0xE9
	
endHex2SioB	dw	low(adrHex2SioB)	;8080 start address
	dw	high(adrHex2SioB)

;----------------------------------------------------------------------------
; Memory test (2SIO as consle)
;----------------------------------------------------------------------------
adrMemTest	equ	0x0000		;load address in 8080

injMemTest	dw	low(adrMemTest)	;8080 load address
	dw	high(adrMemTest)
	dw	endMemTest-strMemTest	;length of block

strMemTest	equ	$

 dw 0xDB,0xFF,0x31,0x00,0x01,0x3E,0x03,0xD3,0x10,0x3E,0x15,0xD3,0x10,0xCD,0x44,0x00
 dw 0x3E,0x2A,0xCD,0x38,0x00,0xC3,0x61,0x00,0x21,0x00,0x00,0x0E,0x04,0xCD,0x4E,0x00
 dw 0x29,0x29,0x29,0x29,0xD6,0x30,0xFE,0x0A,0xDA,0x2F,0x00,0xE6,0xDF,0xD6,0x07,0x85
 dw 0x6F,0x0D,0xC2,0x1D,0x00,0xEB,0x3E,0x20,0xF5,0xDB,0x10,0xE6,0x02,0xCA,0x39,0x00
 dw 0xF1,0xD3,0x11,0xC9,0x3E,0x0D,0xCD,0x38,0x00,0x3E,0x0A,0xC3,0x38,0x00,0xDB,0x10
 dw 0xE6,0x01,0xCA,0x4E,0x00,0xDB,0x11,0xE6,0x7F,0xFE,0x1B,0xCA,0x00,0x00,0xC3,0x38
 dw 0x00,0xCD,0x18,0x00,0xCD,0x18,0x00,0x01,0x5A,0x5A,0xCD,0x97,0x00,0xC5,0xE5,0xD5
 dw 0xCD,0x97,0x00,0x70,0xCD,0xDB,0x00,0xC2,0x70,0x00,0xD1,0xE1,0xC1,0xE5,0xD5,0xCD
 dw 0x97,0x00,0x7E,0xB8,0xC4,0xB5,0x00,0xCD,0xDB,0x00,0xC2,0x7F,0x00,0xD1,0xE1,0x3E
 dw 0x2E,0xCD,0x38,0x00,0xC3,0x6A,0x00,0x78,0xE6,0xB4,0xA7,0xEA,0x9F,0x00,0x37,0x79
 dw 0x17,0x4F,0x78,0x17,0x47,0xC9,0xCD,0x44,0x00,0x7C,0xCD,0xC1,0x00,0x7D,0xCD,0xC1
 dw 0x00,0xCD,0x36,0x00,0xC9,0xF5,0xCD,0xA6,0x00,0x78,0xCD,0xC1,0x00,0xCD,0x36,0x00
 dw 0xF1,0xF5,0xCD,0xC9,0x00,0xF1,0xC3,0xCD,0x00,0x1F,0x1F,0x1F,0x1F,0xE6,0x0F,0xC6
 dw 0x30,0xFE,0x3A,0xDA,0x38,0x00,0xC6,0x07,0xC3,0x38,0x00,0x7B,0x95,0xC2,0xE2,0x00
 dw 0x7A,0x9C,0x23,0xC9
	
endMemTest	dw	low(adrMemTest)	;8080 start address
	dw	high(adrMemTest)

	end

