;*********************************************************************************
;
; Altair 8800c Front Panel Control Program   -- SAFE MODE --
;
;    This file is the "safe mode" version of the front panel control program.
;    It is in protected flash memory. This mode ensures the front panel,
;    and therefore the Altair, can run even if the "normal mode" code is not
;    present, is corrupted, or has a fatal bug. Only the features of the 
;    original Altair front panel are implemented in safe mode. Parameters
;    from EEPROM to enable/disable features are NOT read.
;
;    The safe mode also allows the download and programming of the normal mode
;    firmware. The operator can force entry into safe mode by holding up
;    AUX-RIGHT while powering on the computer. Safe mode can be entered from
;    normal mode by raising RESET while holding AUX-RIGHT in the up position.
;
;    Updating normal mode firmware is accomplished by connecting a PC running
;    a terminal emulator to a 2SIO serial port in the Altair computer (at the
;    standard 10h address). The PIC processor injects a serial I/O bridge
;    into 8080 RAM that allows the PIC to communicate via the 2SIOserial port.
;    Firmware update mode is activated by raising AUX-LEFT while holding
;    AUX-RIGHT in the up position.
;
;    The smallest protected flash block that fits this code is 0x800 words in 
;    length which is half of the 4K program space. However, this code only takes
;    0x400 words of the 0x800 word block, wasting about 0x400 words of program
;    space. To utilize the remaining protected space, code data for a number of
;    the injectable 8080 code segments is stored in this protected space. These
;    are mostly the original Altair bootloaders which won't change.
; 
;    Author: Mike Douglas
; 
;  Revision: 1.0 (12/6/18),  Initial release 
;	 Checksum: 0x15F2
;
;*********************************************************************************
;
;  Processor is a 16F1828, 16 MHz Internal Oscillator w/o PLL
;
	list p=16f1828,b=12,r=dec
	include <p16f1828.inc>

; CONFIG options: Power-on timer is disabled for quicker startup, brown-out set
;   low voltage range (1.9v), no copy protection, watchdog timer is off,
;   0x000-0x7FF protected from self programming.

	__config _CONFIG1, _FCMEN_OFF & _IESO_OFF & _CLKOUTEN_OFF & _BOREN_ON & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDTE_OFF & _FOSC_INTOSC
	__config _CONFIG2, _LVP_OFF & _BORV_19 & _STVREN_OFF & _PLLEN_OFF & _WRT_HALF

; Port A equates		;(initial value) in parentheses

smCYCLEn	equ	5	;(1) low to cycle 8080 one cycle
smRESET	equ	4	;(1) high to assert PRESET to bus
smPOCn	equ	3	;(In) POC* (power on clear) signal from bus
smSERCLK	equ	2	;(0)serial transfer clock 
			;test data in, then positive pulse, LSB first
			;data out, NOP, positive pulse, LSB first
smSERIN	equ	1	;(In) data in from shift registers (front panel switches)
smINLATn	equ	0	;(1) pulse low to latch switches into shift registers

; Port B equates

smDI_LATFFn	equ	7	;(1) high to show data-in bus on LEDs, low shows FF latch
smOUTLAT	equ	6	;(0) pulse high to latch shift reg -> CPU inject latch
smUNPROTn	equ	5	;(1) low to assert UNPROT to bus
smSEROUT	equ	4	;(0) serial data out -> shift reg for CPU inject latch

; Port C equates

smMWRITE	equ	7	;(0) high to assert MWRITE to bus
smFFIN	equ	6	;(In) positive pulse (<500ns) when IN from 0FFh performed
			;   by 8080. Captured by CCP4
smDATAOE	equ	5	;(0) high to enable output of CPU inject latch
smRUN	equ	4	;(0) high to let the 8080 run, low to stop the 8080
smFFOUT	equ	3	;(In) positive pulse (<500ns) when OUT to 0FFh performed
			;   by 8080. Captured by CCP2
smPROTn	equ	2	;(1) low to assert PROT to bus
smSSn	equ	1	;(1) low to assert single-step to bus
smEXTCLR	equ	0	;(0) high to assert EXTCLR to bus

; On the front panel board, five 8-bit serial shift registers are used to read all switches 
;    and the value displayed on the data LEDs. Pulse the smINLATn low to parallel load all
;    five shift registers. Use the serial clock and data (smSERCLK, smSERIN) to read the
;    shift registers. Five bytes come in, LSBit to MSBit in the following order: (note that
;    all five shift registers do NOT have to be read if all data is not required)
;	
;    1) Data LED latch
;    2) A7-A0
;    3) RESET - AUX command switch group (see bit positions below)
;    4) A15-A8
;    5) STOP/RUN - DEPOSIT command switch group (see bit positions below)
;

swSTOP	equ	2	;STOP up		
swRUN	equ	3	;RUN down
swSSTEP	equ	0	;SINGLE STEP up
swSSTEPD	equ	1	;SINGLE STEP down
swEXAMINE	equ	5	;EXAMINE up
swEXAMNEXT	equ	4	;EXAMINE NEXT down
swDEPOSIT	equ	7	;DEPOSIT up
swDEPNEXT	equ	6	;DEPOSIT NEXT down

swRESET	equ	2	;RESET up
swEXTCLEAR	equ	3	;EXTernal CLEAR down
swPROTECT	equ	0	;memory PROTECT up
swUNPROTECT	equ	1	;memory UNPROTECT down
swAUXLEFTU	equ	5	;left AUX up
swAUXLEFTD	equ	4	;left AUX down
swAUXRIGHTU	equ	7	;right AUX up
swAUXRIGHTD	equ	6	;right AUX down

; Linkage equates between safe and normal mode  **DO NOT CHANGE**

SAFE_MODE	equ	0x0000	;address of "safe mode" code
NORMAL_MODE	equ	0x0800	;address of "normal mode" code
INJECT_PTRS	equ	0x0005	;table of pointers to injectable 8080 code
INJECT_PCNT	equ	36	;number of pointers = injPtrsCnt below

; Flags variable equates

fRUNNING	equ	0	;1=CPU running

; Misc Equates

JMP_INST	equ	0xC3	;8080 JMP instruction
NOP_INST	equ	0x00	;8080 NOP instruction

BRIDGE_IDLE	equ	0xAA	;serial port bridge is idle/good
BRIDGE_IHEX	equ	0xA5	;tell bridge to load Intel hex file
BRIDGE_HERR	equ	0xAE	;error loading hex file
PGMDATA_ADR	equ	0x0200	;address of flash program data in 8080

CR	equ	0x0D	;ASCII carriage return
LF	equ	0x0A	;ASCII line feed
BS	equ	0x08	;ASCII back space

INVALID_PTR	equ	0x3FFF	;invalid pointer value


;************************************************************************
;
;  Data Definitions
;
;************************************************************************
	
; Page 0 data
	cblock	0x20	;start of bank 0 RAM
prevA15_A8			;previous value of switches A15 to A8
prevA7_A0			;previous value of switches A7-A0
prevLeft4			;previous value of left four switches
prevRight4			;previous value of right four switches

swCount			;switch processing loop counter
inChar			;byte read from serial bridge
inBuf			;typing input buffer (must be last)
	endc
INBUF_END	equ	0x6E	;last location in inBuf for a char

; Data common to all pages
	cblock	0x70	;common RAM all pages
swA15_A8			;value of switches A15 to A8
swA7_A0			;value of switches A7-A0
swLeft4			;value of left four switches
swRight4			;value of right four switches
flags			;flags byte
shiftReg			;input and output shift register
byteCount			;multi-purpose byte counter
bitCount			;multi-purpose bit counter
delayCount			;delay counter for delay subroutine
	endc

;************************************************************************
;
;  Program Space
;
;************************************************************************

; Reset and interrupt vectors

	org	0
	goto	devReset		;reset vector location zero
	goto	devReset
	goto	devReset
	goto	devReset
	goto	devReset		;interrupt vector location 0x04

;------------------------------------------------------------------------
; injectPtrs - Table of pointers to 8080 code segments that the user
;    can select on the front panel and have injected into RAM. Original
;    bootstrap loaders that will never change are stored here in the 
;    safe mode address space to save room in the updateable section
;    of flash program space.
;
; THIS TABLE CANNOT CROSS A 256 WORD BOUNDARY
;------------------------------------------------------------------------
	org	INJECT_PTRS

; 000
	dw	low(inj5k2sio10),high(inj5k2sio10)	   ;BASIC 1.0
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(inj5kSio10),high(inj5kSio10)
	dw	low(inj5kCas10),high(inj5kCas10)
; 004
	dw	low(inj4k2sio32),high(inj4k2sio32)	   ;4K BASIC 3.2
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(inj4kSio32),high(inj4kSio32)
	dw	low(inj4kCas32),high(inj4kCas32)
; 010
	dw	low(inj8k2sio32),high(inj8k2sio32)	   ;8K BASIC 3.2
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(inj8kSio32),high(inj8kSio32)
	dw	low(inj8kCas32),high(inj8kCas32)
; 014	
	dw	low(inj4k2sio40),high(inj4k2sio40)	   ;4K BASIC 4.0
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(inj4kSio40),high(inj4kSio40)
	dw	low(inj4kCas40),high(inj4kCas40)
; 020
	dw	low(inj8k2sio40),high(inj8k2sio40)	   ;8K BASIC 4.0
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(inj8kSio40),high(inj8kSio40)
	dw	low(inj8kCas40),high(inj8kCas40)
; 024
	dw	low(injEx2sio40),high(injEx2sio40)     ;Ext BASIC 4.0
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(injExtSio40),high(injExtSio40)
	dw	low(injExtCas40),high(injExtCas40)
; 030
	dw	low(injLdrDet32),high(injLdrDet32)	   ;v3.2 cassette leader detector 
	dw	low(injLdrDet40),high(injLdrDet40)	   ;v4.0 cassette leader detector 
	dw	low(INVALID_PTR),high(INVALID_PTR)
	dw	low(INVALID_PTR),high(INVALID_PTR)
; 034
	dw	low(injEco2sioA),high(injEco2sioA)	   ;port echo routines
	dw	low(injEco2sioB),high(injEco2sioB)
	dw	low(injEcoSio),high(injEcoSio)
	dw	low(injEcoCas),high(injEcoCas)
; 040
	dw	low(injKillBit),high(injKillBit)	   ;Kill the Bit
	dw	low(injPong),high(injPong)	   ;Pong
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;Reserved
	dw	low(INVALID_PTR),high(INVALID_PTR)	   ;Reserved

injPtrsCnt	equ	($-INJECT_PTRS)/2

;----------------------------------------------------------------------------
; swJumpTbl - Jump table for command switch processing.
;    Switches are processed in a loop from LSB to MSB for the right group
;    of switches then the same for the left group of switches. swCount
;    decrements from 16 downto 1 for this sequence. As each switch bit is
;    checked, swCount-1 is used as an index into the table below to process
;    the corresponding switch.
;
; THIS TABLE CANNOT CROSS A 256 WORD BOUNDARY
;----------------------------------------------------------------------------
swJumpTbl	goto	doDeposit
	goto	doDepNext	
	goto	doExamine
	goto	doExamNext
	goto	doRun
	goto	doStop
	goto	doAutoStep
	goto	doStep

	goto	doAuxRightU
	goto	doAuxRightD
	goto	doAuxLeftU
	goto	doAuxLeftD
	goto	doExtClear
	goto	doReset
	goto	doUnprotect
	goto	doProtect

; devReset - Device reset entry point. Running at 500khz here. Perform critical
;    initializaton only. With no power-on timer, the processor is running
;    immediately as Vcc crosses just 2v or so. This code may get restarted
;    several times due to power supply glitches during startup.

devReset	equ	$

; Initialize PORT C (do first to get MWRITE driven low ASAP)

	banksel	PORTC
	movlw	b'00000110'		;run/stop flip-flop in stopped state
	movwf	PORTC		;   (though probably won't work)
	banksel	LATC
	movwf	LATC		;init Port C latch = Port C
	banksel	TRISC
	movlw	b'01001000'		;0xFF in/out flags are inputs 
	movwf	TRISC
	banksel	ANSELC
	clrf	ANSELC		;all digital - no analog
	banksel	WPUC
	clrf	WPUC

; Initialize PORT A

	banksel	PORTA
	movlw	b'00100001'		;reset to 8080 not asserted
	movwf	PORTA
	banksel	LATA
	movwf	LATA		;init Port A latch = Port A
	banksel	TRISA
	movlw	b'00001010'		;pwr on clear and serial in are inputs
	movwf	TRISA
	banksel	ANSELA
	clrf	ANSELA		;all digital - no analog
	banksel	WPUA
	clrf	WPUA		;no pull-ups used

; Initialize PORT B

	banksel	PORTB
	movlw	b'10100000'		;data-in bus shows on LEDs
	movwf	PORTB
	banksel	LATB
	movwf	LATB		;init Port B latch = Port B
	banksel	TRISB
	movlw	b'00000000'		;no inputs on Port B
	movwf	TRISB
	banksel	ANSELB
	clrf	ANSELB		;all digital - no analog
	banksel	WPUB
	clrf	WPUB		;no pull-ups used

; Wait up to 0.5s for power-on clear (POC*) from the bus to be de-asserted
;    continuously for 43ms. Running at 500khz here (8us instruction cycle).

	banksel	PORTA		;back to bank 0

	clrf	swCount		;use swCount as POC 43ms counter
	movlw	80		;0.5s = 80 * 6.208ms
	movwf	delayCount		;use delayCount as MSB of delay

chkPoc	btfss	PORTA,smPOCn	;power on clear from bus removed?
	clrf	swCount		;no, restart count

	incf	swCount,F		;increment at 6.208ms rate
	btfsc	swCount,3		;increment to 8 yet? 
 	goto	pwrGood		;yes, power is now good		

pocDelay	decfsz	bitCount,F		;use bitCount for 6.144ms per 256
	goto	pocDelay	

	decfsz	delayCount,F	;decrement MSB of counter
	goto	chkPoc		;go check POC again
	
; Power on delays are complete. Assume it's safe to bump up to 16mhz and run

pwrGood	equ	$
	banksel	OSCCON
	movlw	b'01111000'		;16 Mhz internal clock
	movwf	OSCCON
	nop			;give time to stabilize
	nop
	nop
	nop

; Init OPTION register to be safe. TRM0 runs from instruction clock.

	banksel	OPTION_REG		;init OPTION register
	movlw	b'01010111'		;/256 on TMR0
	movwf	OPTION_REG		;TMR0 wraps at 61hz (not used)

; Set TMR1 to run at 4mhz. Input capture is used to detect when the 8080
;   does an OUT or IN from 0FFh, though time information is not acutally
;   used. Init input capture 2 to come in on RC3 and set both IC2 and IC4
;   to capture the falling edge of the OUT or IN pulse.

	banksel	T1CON
	movlw	b'00000101'		;timer 1 on at 4mhz rate
	movwf	T1CON

	banksel	APFCON1		
	movlw	b'00000000'		;alternate pin function, CCP2 on RC3
	movwf	APFCON1

	movlw	b'00000100'		;capture on falling edge
	banksel	CCP2CON
	movwf	CCP2CON		;for CCP2 (OUT 0FFh)
	banksel	CCP4CON
	movwf	CCP4CON		;and CCP4 (IN 0FFh)

; Set up TMR2 to generate 25ms wakeup for polling front panel switches. The
;   timer increments at 62.5khz (4mhz/64) with a period of 156 (2.496ms).
;   A /10 post-scale gives a 25ms interrupt flag.

	banksel	T2CON
	movlw	b'01001111'		;/64 pre- and /10 post-scale
	movwf	T2CON
	
	clrf	TMR2		;clear timer to zero
	movlw	155		;period is 156/(4mhz/64) = 2.5ms
	movwf	PR2
	
	bcf	PIR1,TMR2IF		;clear the TMR2 interrupt flag

; Initialize current state of all switches, put value of sense switches (A15-A8)
;    into the CPU inject latch.

	banksel	LATA		;access latch registers
	bcf	LATA,smRESET	;release reset
	
	bcf	LATA,smINLATn	;latch all switches into shift registers
	nop			;give time to settle
	bsf	LATA,smINLATn
	
	call	readLatch		;read 1st shift register (ignored)
	
	call	readLatch		;A7-A0 switches
	movwf	prevA7_A0
	
	call	readLatch		;right command switches
	movwf	prevRight4
	
	call	readLatch		;A15-A8 switches
	movwf	prevA15_A8

	call	readLatch		;left command switches
	movwf	prevLeft4

	movfw	prevA15_A8		;sense switches into CPU inject latch
	call	writeLatch

	clrf	flags		;all flags off
	
; Jump to normal mode code at 0x0800 unless the operator requested safe mode by
;    raising AUX-RIGHT during power up, or normal mode flash is not programmed

	btfss	prevRight4,swAUXRIGHTU	;AUX-RIGHT raised?
	goto	pwrReset		;yes, stay in safe mode

	movlw	low(NORMAL_MODE)	;point FSR0 to normal mode address
	movwf	FSR0L
	movlw	high(NORMAL_MODE)+0x80
	movwf	FSR0H
	movwf	PCLATH		;high byte of PC for possible jump

	incf	INDF0,W		;low byte of instruction 0xFF?
	btfss	STATUS,Z
	goto	NORMAL_MODE		;not 0xFF, go execute normal mode

	incf	FSR0,F		;bump to next instruction
	incf	INDF0,W		;low byte of instruction 0xFF?
	btfss	STATUS,Z
	goto	NORMAL_MODE		;not 0xFF, go execute normal mode

	clrf	PCLATH		;remain here in safe mode

pwrReset	equ	$
	banksel	LATA		;access latch registers
	bsf	LATA,smRESET	;assert reset
	movlw	250		;hold for 250us
	call	delayUs
	bcf	LATA,smRESET	;de-assert reset
	banksel	PORTA		;return to bank 0
				;fall into main loop
			
;****************************************************************************
;
; Main Loop
;     Every 25ms, read all switches and take the appropriate actions.
;
;****************************************************************************
main	btfss	PIR1,TMR2IF		;wait for 25ms flag from timer 2
	goto	main

	bcf	PIR1,TMR2IF		;clear the TMR2 interrupt flag
	call	rdSwitches		;load current state of all switches

chkSwitches	movfw	swLeft4		;check left 4 command switches
	xorwf	prevLeft4,F		;changed switches will be 1	
	movfw	swRight4		;check right 4 command switches
	xorwf	prevRight4,F	;changed switches will be 1

	movlw	high(swJumpTbl)	;set MSB of jump table
	movwf	PCLATH

; Loop through all 16 command switches and call the appropriate process routine
;    for any switch that has changed state. Switches are processed from LSB to
;    MSB for the right group of switches then the same for the left group of
;    switches. swCount decrements from 16 downto 1 for this sequence. As each
;    bit is checked, swCount-1 is used as an index into swJumpTbl to call the
;    proper subroutine to process the switch.

	movlw	16		;checking 16 switch states
	movwf	swCount
	
swLoop	btfss	prevRight4,0	;did switch change?
	goto	noChange		;no
	
	movlw	low(swJumpTbl-1)	;compute LSB of jump address
	addwf	swCount,W
	callw			;call appropriate switch processor
	
noChange	lsrf	prevLeft4,F		;shift MSByte
	rrf	prevRight4,F	;shift LSByte
	decfsz	swCount,F
	goto	swLoop

; Move current to previous switch state, put sense switches into CPU inject latch

	movfw	swLeft4		;move current to previous
	movwf	prevLeft4
	movfw	swRight4
	movwf	prevRight4

	movfw	swA15_A8		;sense switches into CPU inject latch
	call	writeLatch

	goto	main		;do it all over again
	
;----------------------------------------------------------------------------
; doStop - Stop switch raised or released. If raised, set smRUN low to
;    stop the 8080 in a wait state. Force display of the data-in bus on
;    the LEDs. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doStop	btfsc	swLeft4,swSTOP	;switch raised or released?
	return			;released, exit
	
	bcf	flags,fRUNNING	;8080 is stopped 
	banksel	LATA		;select latches bank
	bcf	LATC,smRUN		;clear smRUN to stop the 8080
	call	delay10		;delay 10us to ensure completion
	
	banksel	PORTA		;return to bank 0
	return
	
;----------------------------------------------------------------------------
; doRun - Run switch depressed or released. If depressed, set smRUN high
;    to let the 8080 run. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doRun	btfsc	swLeft4,swRUN	;switch depressed or released?
	return			;released, exit
	
	bsf	flags,fRUNNING	;8080 is running
	banksel	LATA		;select latches bank
	bsf	LATC,smRUN		;set smRUN high
	call	delay10		;delay 10us to ensure completion

	banksel	PORTA		;return to bank 0
	return

;----------------------------------------------------------------------------
; doStep - Step switch raised or released. If raised, assert single-step
;    to the CPU board and then smCYCLE to let the 8080 execute one machine
;    cycle. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doStep	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	

	btfsc	swLeft4,swSSTEP	;switch raised or released?
	return			;released, exit
	
	banksel	LATA		;select latches bank
	bcf	LATC,smSSn		;assert single-step to the CPU board
	bcf	LATA,smCYCLEn	;let 8080 execute one machine cycle
	nop			;assert for 750ns
	nop
	bsf	LATA,smCYCLEn	;8080 will stop after next PSYNC
	call	delay10		;delay 10us to ensure completion
	bsf	LATC,smSSn		;de-assert single-step
	
	banksel	PORTA		;return to bank 0
	return

;----------------------------------------------------------------------------
; doAutoStep - Step switch depressed or released. Do nothing.
;
; BankSel on exit: no change
;----------------------------------------------------------------------------
doAutoStep	return


;----------------------------------------------------------------------------
; doExamine - Examine switch raised or released. If raised, inject the
;    three cycles of a JMP instrucion with the jump address specified 
;    on the front panel address switches. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doExamine	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	
	
	btfsc	swLeft4,swEXAMINE	;switch raised or released?
	return			;released, exit

	movlw	JMP_INST		;inject JMP opcode to CPU
	call	injectW	
	movfw	swA7_A0		;inject low byte of address
	call	injectW	
	movfw	swA15_A8		;inject high byte of address
	call	injectW

	return

;----------------------------------------------------------------------------
; doExamNext- Examine Next switch depressed or released. If depressed,
;    inject a NOP instruction to cause the CPU to go to the next address.
;    No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doExamNext	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	

	btfsc	swLeft4,swEXAMNEXT	;switch depressed or released?
	return			;released, exit
	
	movlw	NOP_INST		;inject NOP instruction to CPU
	call	injectW
	
	return

;----------------------------------------------------------------------------
; doDeposit - Deposit switch raised or released. If raised, deposit the
;     the value on switches A7-A0 into the current address by asserting
;     MWRITE. If AUX-RIGHT is also raised, then consider this a request
;     to enter firmware update mode. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doDeposit	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	

	btfsc	swLeft4,swDEPOSIT	;switch raised or released?
	return			;released, exit

doDepositEx	movfw	swA7_A0		;W=value to deposit
	call	writeLatch		;write W to inject latch
	banksel	LATA		;select latches bank
	bsf	LATC,smDATAOE	;enable output of inject latch
	nop			;let data settle

	bsf	LATC,smMWRITE	;assert MWRITE to write data to RAM.
	movlw	8		;assert for 8us like original Altair
	call	delayUs
	bcf	LATC,smMWRITE	;remove MWRITE
	
	nop			;give some hold time
	bcf	LATC,smDATAOE	;disable output of CPU inject latch

	banksel	PORTA		;return to bank 0
	return
	
;----------------------------------------------------------------------------
; doDepNext- Deposit Next switch depressed or released. If depressed,
;    inject a NOP instruction to cause the CPU to go to the next address,
;    then jump to doDeposit to write the value on switches A7-A0 to
;    memory. No action on release.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doDepNext	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	

	btfsc	swLeft4,swDEPNEXT	;switch depressed or released?
	return			;released, exit
	
	movlw	NOP_INST		;inject NOP instruction to CPU
	call	injectW	
	movlw	250		;delay 500us after 8080 NOP
	call	delayUs		;  to duplicate original Altair
	movlw	250		;  timing. This causes a slight
	call	delayUs		;  LED flicker during deposit next
	
	goto	doDepositEx		;deposit value on switches
	
;----------------------------------------------------------------------------
; doReset - Reset switch raised or released. If raised, assert smRESET.
;    If released, de-assert smRESET.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
doReset	equ	$
	banksel	LATA		;access latch registers

	btfss	swRight4,swRESET	;switch raised?
	bsf	LATA,smRESET	;yes, assert reset

	btfsc	swRight4,swRESET	;switch released?
	bcf	LATA,smRESET	;yes, de-assert reset
			
	banksel	PORTA		;return to bank 0
	return	
		
;----------------------------------------------------------------------------
; doExtClear - External Clear switch depressed or released. If depressed,
;    assert smEXTCLR. If released, de-assert smEXTCLR.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
doExtClear	equ	$
	banksel	LATA		;access latch registers

	btfss	swRight4,swEXTCLEAR	;switch depressed?	
	bsf	LATC,smEXTCLR	;yes, assert external clear

	btfsc	swRight4,swEXTCLEAR	;switch released?
	bcf	LATC,smEXTCLR	;yes, de-assert external clear

	banksel	PORTA		;return to bank 0
	return

;----------------------------------------------------------------------------
; doProtect - Protect switch raised or released. If raised and the CPU is
;    not running, then assert smPROT. If released, always de-assert 
;    smPROT just to be safe.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doProtect	btfsc	swRight4,swPROTECT	;switch raised or released?
	goto	dpRelease		;released

	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, exit

	banksel	LATA		;access latch registers
	bcf	LATC,smPROTn	;assert protect signal
	
	banksel	PORTA		;return to bank 0
	return

dpRelease	equ	$
	banksel	LATA		;access latch registers
	bsf	LATC,smPROTn	;turn off protect signal
	
	banksel	PORTA		;return to bank 0
	return

;----------------------------------------------------------------------------
; doUnprotect - Unprotect switch depressed or released. If depressed
;    and the CPU is not running, then assert smUNPROT. If released,
;    always de-assert smUNPROT just to be safe.
;
; BankSel on exit: no change or bank zero
;----------------------------------------------------------------------------
doUnprotect	btfsc	swRight4,swUNPROTECT	;switch depressed or released?
	goto	duRelease		;released

	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, exit

	banksel	LATA		;access latch registers
	bcf	LATB,smUNPROTn	;assert unprotect signal
	
	banksel	PORTA		;return to bank 0
	return

duRelease	equ	$
	banksel	LATA		;access latch registers
	bsf	LATB,smUNPROTn	;turn off unprotect signal
	
	banksel	PORTA		;return to bank 0
	return
	
;----------------------------------------------------------------------------
; doAuxLeftU - Left Aux switch raised or released. If raised, see if
;    Aux-Right is also raised to request entry into firmware update
;    mode. If released, do nothing.
;
; BankSel on exit: no change
;----------------------------------------------------------------------------
doAuxLeftU	btfsc	flags,fRUNNING	;CPU running?
	return			;yes, ignore the switch	

	btfsc	swRight4,swAUXLEFTU	;switch raised or released?
	return			;released, exit

	btfsc	swRight4,swAUXRIGHTU	;AUX-RIGHT also raised?
	return			;no, ignore

	movfw	swLeft4		;move current to previous
	movwf	prevLeft4		;  so main loop can be
	movfw	swRight4		;  properly re-entered 
	movwf	prevRight4		;  later

	goto	programMode		;enter program mode

;----------------------------------------------------------------------------
; doAuxLeftD - Left Aux switch depressed or released. Do nothing.
;
; BankSel on exit: no change
;----------------------------------------------------------------------------
doAuxLeftD	return


;----------------------------------------------------------------------------
; doAuxRightU - Right Aux switch raised or released. Do nothing.
;
; BankSel on exit: no change
;----------------------------------------------------------------------------
doAuxRightU	return

	
;----------------------------------------------------------------------------
; doAuxRightD - Right Aux switch depressed or released. Do nothing.
;
; BankSel on exit: no change
;----------------------------------------------------------------------------
doAuxRightD	return	


;----------------------------------------------------------------------------
; rdSwitches - Read the current state of all switches from the shift 
;   registers into the corresponding switch variables. Takes 121.75us
;   (487 cycles) including call and return.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
rdSwitches	equ	$
	banksel	LATA		;access latch registers

	bcf	LATA,smINLATn	;latch all switches into shift registers
	nop			;time to settle over ribbon cable
	bsf	LATA,smINLATn

	call	readLatch		;read 1st shift register (ignored)

	call	readLatch		;A7-A0 switches
	movwf	swA7_A0

	call	readLatch		;right action switches
	movwf	swRight4

	call	readLatch		;A15-A8 switches
	movwf	swA15_A8

	call	readLatch		;left action switches
	movwf	swLeft4

	return

;----------------------------------------------------------------------------
; injectW - Put the value passed in W into the CPU inject latch and
;    then assert smCYCLEn to let the 8080 run a machine cycle.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
injectW	call	writeLatch		;write W to CPU inject latch

	banksel	LATA		;access latch registers
	bsf	LATC,smDATAOE	;enable output of CPU inject latch
	
	bcf	LATA,smCYCLEn	;let 8080 execute one machine cycle
	nop			;assert for 750ns
	nop
	bsf	LATA,smCYCLEn	;8080 will stop after next PSYNC
	call	delay10		;give some time to finish

	bcf	LATC,smDATAOE	;disable output of CPU inject latch

	banksel	PORTA		;exit with bank 0 selected
	return
	
;----------------------------------------------------------------------------
; readLatch - Read the next 8 bits from the 40-bit shift register latch
;    and return in W. Data is read LSBit first. Takes 23.75us (95 cycles)
;    including call and return.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
readLatch	movlw	8		;reading 8 bits
	movwf	bitCount

rlLoop	lsrf	shiftReg,F		;shift towards LSB, bit 7 is 0
	banksel	PORTA		;access ports for input
	btfsc	PORTA,smSERIN	;test data bit
	bsf	shiftReg,7		;it was one

	banksel	LATA		;access latch registers
	bsf	LATA,smSERCLK	;clock the shift registers
	nop			;time to settle over ribbon cable
	bcf	LATA,smSERCLK

	decfsz	bitCount,F		;repeat for 8 bits
	goto	rlLoop

	movfw	shiftReg		;return result in W	
	banksel	PORTA		;return to bank 0
	return
	
;----------------------------------------------------------------------------
; writeLatch - Write the byte from W to the CPU inject latch. Data is
;    is first written to a shift register and then latched into an 8-bit
;    latch. Data is shifted out LSBit first. Takes 26.5us (106 cycles)
;    including call and return.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
writeLatch	movwf	shiftReg		;save value to write
	movlw	8		;writing 8 bits
	movwf	bitCount
	banksel	LATA		;access latch registers

wlLoop	btfss	shiftReg,0		;what is next data bit?
	goto	wlData0		;bit is zero
	
	bsf	LATB,smSEROUT	;bit is one
	goto	wlClock		;go clock in the bit
	
wlData0	bcf	LATB,smSEROUT	;bit is zero
	nop			;data setup time

wlClock	bsf	LATA,smSERCLK	;clock the shift register
	nop			;clock is also on ribbon cable
	bcf	LATA,smSERCLK

	lsrf	shiftReg,F		;move to next bit	
	decfsz	bitCount,F		;repeat for 8 bits
	goto	wlLoop
	
	bsf	LATB,smOUTLAT	;latch shift register to parallel latch
	bcf	LATB,smOUTLAT
	
	banksel	PORTA		;exit with bank 0 selected
	return

;----------------------------------------------------------------------------
; delay10 - Delay 10us
; delayUs - Delay W us including movlw n, call, and return. Minimum
;     delay is 3us.
;
; BankSel on exit: no change
;----------------------------------------------------------------------------
delay10	movlw	10		;delay 10us

delayUs	addlw	-2		;account for fixed overhead
	movwf	delayCount		;save delay count
	nop
	
duLoop	nop
	decfsz	delayCount,F
	goto	duLoop
	
	nop
	return

;----------------------------------------------------------------------------
; programMode - Firmware update (program) mode
;
;    The program mode is entered by raising DEPOSIT while holding the
;    AUX-RIGHT switch in the up position. (i.e., to "Deposit" new firmware
;    into flash memory)
;
;    The program mode performs console I/O through a standard 2SIO port
;    (at 10h) in the host machine. The 2SIO is accessed via a serial port
;    bridge program that is injected into host memory at 80h by this code
;    and is then executed by the 8080.
;
;    The program mode is exited based on operator response to prompts or
;    by a front panel RESET.
;----------------------------------------------------------------------------
programMode	movlw	low(injSioBrdg)	;point FSR0 to 8080 bridge code
	movwf	FSR0L
	movlw	high(injSioBrdg)
	movwf	FSR0H
	call	depositBlk		;put the program in 8080 RAM

	movlw	BRIDGE_IDLE		;give IDLE state to 8080 through IN FF
	call	writeLatch
	
	banksel	LATA		;select latches bank
	bsf	LATC,smRUN		;set smRUN high to let 8080 run
	
	banksel	PIR2		;select PIR2 bank (bank zero)
	bcf	PIR2,CCP2IF		;clear the OUT FF detect flag

; Wait for up to 25ms for 8080 code to say it is ready by executing an
;    OUT 0FFh. If timeout occurs, resume normal operation.

waitBridge	btfsc	PIR2,CCP2IF		;wait for 8080 to signal ready
	goto	bridgeRdy		;  by doing an OUT to FFh
	
	btfss	PIR1,TMR2IF		;25ms elapsed?
	goto	waitBridge		;not yet
	
pgmExit	equ	$
	banksel	LATA		;select latches bank
	bcf	LATC,smRUN		;set smRUN low to stop 8080
	banksel	PORTA		;select bank 0	
	goto	main		;back to front panel mode
	
; bridgeRdy - SIO bridge loaded, running, and ready. Prompt user to send the
;    hex file containing the normal mode firmware.
 
bridgeRdy	movlw	low(mPrompt)	;display the file send prompt
	movwf	FSR0L
	movlw	high(mPrompt)
	movwf	FSR0H
	call	dispMsg
		
	movlw	BRIDGE_IHEX		;tell bridge to load hex file
	call	writeLatch
	call	getChar		;wait for completion
	
	addlw	-BRIDGE_IDLE	;hex file read without error?
	btfsc	STATUS,Z
	goto	doProgram		;yes, go program flash

	addlw	BRIDGE_IDLE		;recover original status value
	addlw	-BRIDGE_HERR	;hex file error?
	btfsc	STATUS,Z		
	goto	hexError		;yes, display hex file error

pgmAbort	movlw	low(mAbort)		;display abort message
	movwf	FSR0L
	movlw	high(mAbort)
	goto	msgExit		;exit with message
	
hexError	movlw	low(mHexError)	;display hex file error messgae
	movwf	FSR0L
	movlw	high(mHexError)

msgExit	movwf	FSR0H		;save MSB of exit messsage
	call	dispMsg		;display the exit message
	goto	pgmExit		;exit program mode

; doProgram - New software is in 8080 memory, program flash

doProgram	movlw	low(mProgram)	;display 'Programming...'
	movwf	FSR0L
	movlw	high(mProgram)
	movwf	FSR0H
	call	dispMsg

	call	pgmFlash		;perform the programming
	
	movlw	low(mComplete)	;display 'Programming complete'
	movwf	FSR0L
	movlw	high(mComplete)
	movwf	FSR0H
	call	dispMsg
	goto	devReset		;reboot the front panel

;----------------------------------------------------------------------------
; dispMsg - Display the null terminated string pointed to by FSR0.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
dispMsg	moviw	FSR0++		;get next byte to display
	btfsc	STATUS,Z		;terminating zero?
	goto	setIdle		;yes, go set idle state
	
	call	writeLatch		;give 8080 byte to write
	bcf	PIR2,CCP2IF		;clear the OUT FF detect flag	

dmWait	btfss	PIR2,CCP2IF		;wait for 8080 to signal ready
	goto	dmWait		;  by doing an OUT FF

	goto	dispMsg

;----------------------------------------------------------------------------
; dispChar - Display the character passed in W
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
dispChar	call	writeLatch		;give 8080 byte to write
	bcf	PIR2,CCP2IF		;clear the OUT FF detect flag	

dcWait	btfss	PIR2,CCP2IF		;wait for 8080 to signal ready
	goto	dcWait		;  by doing an OUT FF
	
setIdle	movlw	BRIDGE_IDLE
	call	writeLatch
	
	return
	
;----------------------------------------------------------------------------
; getChar - Get the next character received from the console. While
;    waiting for input, the front panel switches are checked every 25ms
;    to look for a reset abort request. If abort is requested, the 8080
;   is stopped and control returns to normal front panel operation.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
getChar	bcf	PIR2,CCP2IF		;clear the OUT FF detect flag	

gcWait	btfsc	PIR2,CCP2IF		;new char from 8080?
	goto	gcHaveChar		;yes, go process it
	btfss	PIR1,TMR2IF		;25ms up yet?
	goto	gcWait		;no, keep waiting

	bcf	PIR1,TMR2IF		;clear the TMR2 interrupt flag

	call	rdSwitches		;load current state of all switches
	btfsc	swRight4,swRESET	;reset raised?
	goto	gcWait		;no
	
	bcf	swLeft4,swSTOP	;force stop switch activation too
	goto	chkSwitches		;exit to main to process stop-reset
	
gcHaveChar	equ	$
	banksel	LATA		;access latch registers
	bcf	LATB,smDI_LATFFn	;display OUT FF latch from 8080
				;  (allows reading the OUT FF latch)
	bcf	LATA,smINLATn	;latch OUT FF latch into shift register
	nop			;time to settle over ribbon cable
	bsf	LATA,smINLATn
	bsf	LATB,smDI_LATFFn	;restore normal LED display

	call	readLatch		;read character from OUT FF latch
	return			;select bank 0

;----------------------------------------------------------------------------
; depositBlk - Deposit a block of data pointed to by FSR0 into 8080
;   memory. On exit, an "examine" of the specified start address is done
;   to set the 8080 PC to the start address. A data block is organized
;   as follows:
;
;       16 bit load address (LSB, MSB)
;       8 bit length (256 byte maximum, 0=256 bytes)
;       1-256 bytes of 8080 code
;       16 bit start address (LSB, MSB)
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
depositBlk	call	examFSR0		;do "examine" of load address

	moviw	FSR0++		;w=number of bytes to move
	movwf	byteCount

dbLoop	moviw	FSR0++		;w=next byte of 8080 program
	call	writeLatch		;put the byte in the inject latch
	
	banksel	LATA		;select latches bank
	bsf	LATC,smDATAOE	;enable output of inject latch
	nop			;let data settle
	
	bsf	LATC,smMWRITE	;assert MWRITE to write data to RAM.
	movlw	8		;assert for 8us like original Altair
	call	delayUs
	bcf	LATC,smMWRITE	;remove MWRITE
	
	movlw	NOP_INST		;NOP to increment 8080 address
	call	injectW
	
	decfsz	byteCount,F		;repeat for all bytes in block
	goto	dbLoop

; Block of data has been loaded into 8080 RAM. Examine the specified start
;    address in case the caller wants to execute the code.

examFSR0	movlw	JMP_INST		;do "examine" of start address
	call	injectW
	moviw	FSR0++		;lsb of address
	call	injectW	
	moviw	FSR0++		;msb of address
	call	injectW

	return

;----------------------------------------------------------------------------
; pgmFlash - Program 0800-0FFF of flash memory from data in 8080 RAM.
;
; BankSel on exit: bank zero
;----------------------------------------------------------------------------
pgmFlash	equ	$
	banksel	LATA		;select latches bank
	bcf	LATC,smRUN		;set smRUN low to stop the 8080
	
	movlw	JMP_INST		;"examine" program data address in 8080
	call	injectW
	movlw	low(PGMDATA_ADR)	;lsb of address
	call	injectW	
	movlw	high(PGMDATA_ADR)	;msb of address
	call	injectW
	
	banksel	EEADRL
	movlw	low(NORMAL_MODE)	;lsb of flash start address
	movwf	EEADRL
	movlw	high(NORMAL_MODE)	;msb of flash start address
	movwf	EEADRH
	
	bsf	EECON1,EEPGD	;point to program flash
	bcf	EECON1,CFGS		;not configuration words
	bsf	EECON1,WREN		;enable writes

; Erase and program by rows, 32 words at a time. 
;    First, erase a row.

pgmRow	bsf	EECON1,FREE		;specify erase
	movlw	0x55		;write enable sequence...
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR		;set WR to begin erase
	nop			;processor skips this
	nop			;processor stalls here

; Now program the 32 word row

	bsf	EECON1,LWLO		;loading write latches only

prLoadLoop	call	read8080		;load LSB from 8080 RAM
	movwf	EEDATL
	call	read8080		;load MSB from 8080 RAM
	movwf	EEDATH

	movfw	EEADRL		;on 32nd latch? (0x1F)
	andlw	0x1F
	sublw	0x1F
	btfsc	STATUS,Z
	goto	prDoWrite		;last latch - do the write

	movlw	0x55		;write enable sequence...
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR		;set WR to begin erase
	nop			;processor skips this
	nop			;processor stalls here

	incf	EEADRL,F		;move to next word
	goto	prLoadLoop

prDoWrite	bcf	EECON1,LWLO		;no longer loading latches

	movlw	0x55		;write enable sequence...
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR		;set WR to begin erase
	nop			;processor skips this
	nop			;processor stall here

	incfsz	EEADRL,F		;increment flash pointer lsb
	goto	pgmRow		;do next row
	
	incf	EEADRH,F		;increment msb	
	btfss	EEADRH,4		;reach 0x1000?
	goto	pgmRow		;no, continue
	
; Erase the EEPROM to all zeros 

	clrf	EEDATL		;writing zeros (EEADRL already zero)
	bcf	EECON1,EEPGD	;point to EEPROM
	
ceWrite	movlw	0x55		;write enable sequence...
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR		;set WR to start the write
	
ceWait	btfsc	EECON1,WR		;wait for WR to clear
	goto	ceWait
	
	incfsz	EEADRL,F		;increment EEPROM address
	goto	ceWrite		;and write next location
	
	bcf	EECON1,WREN		;EEPROM erased, disable writes

; Programming done. Restart execution of the SIO bridge program on the 8080 and exit.

	movlw	low(endSioBrdg)	;FSR0->SIO bridge start address
	movwf	FSR0L		;lsb
	movlw	high(endSioBrdg)
	movwf	FSR0H		;msb

	call	examFSR0		;do "examine" of SIO bridge address

	banksel	LATA		;select latches bank
	bsf	LATC,smRUN		;set smRUN high to start 8080

	banksel	PORTA		;exit with bank 0 selected
	bcf	PIR2,CCP2IF		;clear the OUT FF detect flag	

pfWait	btfss	PIR2,CCP2IF		;wait for 8080 to signal ready
	goto	pfWait		;  by doing an OUT FF

	return

;----------------------------------------------------------------------------
; read8080 - Read and return the byte currently shown on the data LEDs.
;   Increment the 8080 address by one after reading the current data
;   byte. Exit with the flash control register bank selected
;
; BankSel on exit: EExxx flash control registers bank
;----------------------------------------------------------------------------
read8080	equ	$
	banksel	LATA		;access latch registers
	bcf	LATA,smINLATn	;latch data LEDs into shift register
	nop			;time to settle over ribbon cable
	bsf	LATA,smINLATn

	call	readLatch		;read the 8080 data byte
	movwf	inChar		;save the byte read
	
	movlw	NOP_INST		;NOP to increment 8080 address
	call	injectW
	
	movfw	inChar		;return with data byte in W
	banksel	EEADRL		;exit with flash register bank selected
	return
	
;----------------------------------------------------------------------------
; Message constants
;----------------------------------------------------------------------------
mPrompt	dtm	CR,LF,LF,"Firmware Update, SMV1.0"
	dtm	CR,LF,"Send hex file now (ctrl-c to abort): ",0
mProgram	dtm	CR,LF,LF,"Programming...",0
mComplete	dtm	CR,LF,LF,"Programming complete, restarting in "
	dtm	"the new firmware",CR,LF,0
mHexError	dtm	CR,LF,LF,"** Hex file error **"
mAbort	dtm	CR,LF,LF,"Restarting without update",CR,LF,0

;----------------------------------------------------------------------------
; SIO Bridge - 8080 data block of the SIO bridge program.
;    This includes the Intel hex loader.
;----------------------------------------------------------------------------
adrSioBrdg	equ	0x0080		;load address in 8080

injSioBrdg	dw	low(adrSioBrdg)	;8080 load address
	dw	high(adrSioBrdg)
	dw	endSioBrdg-strSioBrdg	;length of block

strSioBrdg	equ	$

 dw 0xF3,0x31,0x80,0x00,0x3E,0x03,0xD3,0x10,0x3E,0x11,0xD3,0x10,0x3E,0xAA,0xD3,0xFF
 dw 0x3E,0x0D,0x3D,0xC2,0x92,0x00,0xDB,0xFF,0x47,0xDB,0xFF,0xB8,0xC2,0x96,0x00,0xFE
 dw 0xA5,0xCA,0xC3,0x00,0xFE,0xAA,0xC2,0xB6,0x00,0xDB,0x10,0x0F,0xD2,0x96,0x00,0xDB
 dw 0x11,0xD3,0xFF,0xC3,0x96,0x00,0xDB,0x10,0xE6,0x02,0xCA,0xB6,0x00,0x78,0xD3,0x11
 dw 0xC3,0x8C,0x00,0xAF,0x32,0x79,0x01,0x3E,0xAA,0x32,0x7B,0x01,0x21,0x00,0x02,0x01
 dw 0xFF,0x3F,0x16,0x12,0x71,0x23,0x70,0x23,0x7A,0xBC,0xC2,0xD4,0x00,0xCD,0x61,0x01
 dw 0xC2,0xDD,0x00,0x57,0x3E,0x2E,0xD3,0x11,0xCD,0x51,0x01,0x7B,0xB7,0xCA,0x41,0x01
 dw 0x43,0xCD,0x51,0x01,0x63,0xCD,0x51,0x01,0x6B,0xCD,0x51,0x01,0x7B,0xFE,0x04,0xC2
 dw 0x11,0x01,0xCD,0x51,0x01,0x43,0xCD,0x51,0x01,0x7B,0xB0,0x32,0x79,0x01,0xC3,0xDD
 dw 0x00,0x3A,0x79,0x01,0xB7,0xC2,0xDD,0x00,0x58,0x01,0x00,0xF0,0x09,0xD2,0xDD,0x00
 dw 0x01,0x00,0x02,0x09,0x43,0xCD,0x51,0x01,0x73,0x7E,0xBB,0xC2,0x39,0x01,0x23,0x05
 dw 0xC2,0x25,0x01,0xCD,0x51,0x01,0xCA,0xDD,0x00,0x3E,0xAE,0x32,0x7B,0x01,0xC3,0xDD
 dw 0x00,0xDB,0x11,0x1B,0x7A,0xB3,0xC2,0x41,0x01,0x3A,0x7B,0x01,0xD3,0xFF,0xC3,0x90
 dw 0x00,0xCD,0x61,0x01,0x87,0x87,0x87,0x87,0x5F,0xCD,0x61,0x01,0x83,0x5F,0x82,0x57
 dw 0xC9,0xDB,0x10,0x0F,0xD2,0x61,0x01,0xDB,0x11,0xFE,0x03,0xCA,0x4C,0x01,0xD6,0x3A
 dw 0xC8,0xC6,0x0A,0xFE,0x0A,0xD8,0xD6,0x07,0xC9
	
endSioBrdg	dw	low(adrSioBrdg)	;8080 start address
	dw	high(adrSioBrdg)

;----------------------------------------------------------------------------
; BASIC v1.0 bootstrap loader for 88-2SIO
;----------------------------------------------------------------------------
adr5k2sio10	equ	0x1800		;load address in 8080

inj5k2sio10	dw	low(adr5k2sio10)	;8080 load address
	dw	high(adr5k2sio10)
	dw	end5k2sio10-str5k2sio10	;length of program

str5k2sio10	equ	$

 dw 0x3E,0x03,0xD3,0x10,0x3E,0x11,0xD3,0x10,0x26,0x00,0x31,0x1A,0x18,0xDB,0x10,0x0F
 dw 0xD0,0xDB,0x11,0x77,0x23,0xD6,0x6A,0xC0,0x6F,0xC9,0x0A,0x18

end5k2sio10	dw	low(adr5k2sio10)	;8080 start address
	dw	high(adr5k2sio10)

;----------------------------------------------------------------------------
; BASIC v1.0 bootstrap loader for 88-SIO
;----------------------------------------------------------------------------
adr5kSio10	equ	0x1800		;load address in 8080

inj5kSio10	dw	low(adr5kSio10)	;8080 load address
	dw	high(adr5kSio10)
	dw	end5kSio10-str5kSio10	;length of program

str5kSio10	equ	$

 dw 0x26,0x00,0x31,0x12,0x18,0xDB,0x00,0x0F,0xD8,0xDB,0x01,0x77,0x23,0xD6,0x6A,0xC0
 dw 0x6F,0xC9,0x02,0x18

end5kSio10	dw	low(adr5kSio10)	;8080 start address
	dw	high(adr5kSio10)

;----------------------------------------------------------------------------
; BASIC v1.0 bootstrap loader for Cassette
;----------------------------------------------------------------------------
adr5kCas10	equ	0x1800		;load address in 8080

inj5kCas10	dw	low(adr5kCas10)	;8080 load address
	dw	high(adr5kCas10)
	dw	end5kCas10-str5kCas10	;length of program

str5kCas10	equ	$

 dw 0x26,0x00,0x31,0x12,0x18,0xDB,0x06,0x0F,0xD8,0xDB,0x07,0x77,0x23,0xD6,0x6A,0xC0
 dw 0x6F,0xC9,0x02,0x18

end5kCas10	dw	low(adr5kCas10)	;8080 start address
	dw	high(adr5kCas10)

;----------------------------------------------------------------------------
; 4K BASIC v3.2 bootstrap loader for 88-2SIO
;----------------------------------------------------------------------------
adr4k2sio32	equ	0x0000		;load address in 8080

inj4k2sio32	dw	low(adr4k2sio32)	;8080 load address
	dw	high(adr4k2sio32)
	dw	end4k2sio32-str4k2sio32	;length of program

str4k2sio32	equ	$
 dw 0x3E,0x03,0xD3,0x10,0x3E,0x15,0xD3,0x10,0x21,0xAE,0x0F,0x31,0x1A,0x00,0xDB,0x10
 dw 0x0F,0xD0,0xDB,0x11,0xBD,0xC8,0x2D,0x77,0xC0,0xE9,0x0B,0x00

end4k2sio32	dw	low(adr4k2sio32)	;8080 start address
	dw	high(adr4k2sio32)

;----------------------------------------------------------------------------
; 4K BASIC v3.2 bootstrap loader for 88-SIO
;----------------------------------------------------------------------------
adj4kSio32	equ	0x0000		;load address in 8080

inj4kSio32	dw	low(adj4kSio32)	;8080 load address
	dw	high(adj4kSio32)
	dw	end4kSio32-str4kSio32	;length of program

str4kSio32	equ	$

 dw 0x21,0xAE,0x0F,0x31,0x12,0x00,0xDB,0x00,0x0F,0xD8,0xDB,0x01,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

end4kSio32	dw	low(adj4kSio32)	;8080 start address
	dw	high(adj4kSio32)

;----------------------------------------------------------------------------
; 4K BASIC v3.2 bootstrap loader for Cassette
;----------------------------------------------------------------------------
adr4kCas32	equ	0x0000		;load address in 8080

inj4kCas32	dw	low(adr4kCas32)	;8080 load address
	dw	high(adr4kCas32)
	dw	end4kCas32-str4kCas32	;length of program

str4kCas32	equ	$

 dw 0x21,0xAE,0x0F,0x31,0x12,0x00,0xDB,0x06,0x0F,0xD8,0xDB,0x07,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

end4kCas32	dw	low(adr4kCas32)	;8080 start address
	dw	high(adr4kCas32)


;----------------------------------------------------------------------------
; 8K BASIC v3.2 bootstrap loader for 88-2SIO
;----------------------------------------------------------------------------
adr8k2sio32	equ	0x0000		;load address in 8080

inj8k2sio32	dw	low(adr8k2sio32)	;8080 load address
	dw	high(adr8k2sio32)
	dw	end8k2sio32-str8k2sio32	;length of program

str8k2sio32	equ	$

 dw 0x3E,0x03,0xD3,0x10,0x3E,0x15,0xD3,0x10,0x21,0xAE,0x1F,0x31,0x1A,0x00,0xDB,0x10
 dw 0x0F,0xD0,0xDB,0x11,0xBD,0xC8,0x2D,0x77,0xC0,0xE9,0x0B,0x00

end8k2sio32	dw	low(adr8k2sio32)	;8080 start address
	dw	high(adr8k2sio32)

;----------------------------------------------------------------------------
; 8K BASIC v3.2 bootstrap loader for 88-SIO
;----------------------------------------------------------------------------
adr8kSio32	equ	0x0000		;load address in 8080

inj8kSio32	dw	low(adr8kSio32)	;8080 load address
	dw	high(adr8kSio32)
	dw	end8kSio32-str8kSio32	;length of program

str8kSio32	equ	$

 dw 0x21,0xAE,0x1F,0x31,0x12,0x00,0xDB,0x00,0x0F,0xD8,0xDB,0x01,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

end8kSio32	dw	low(adr8kSio32)	;8080 start address
	dw	high(adr8kSio32)

;----------------------------------------------------------------------------
; 8K BASIC v3.2 bootstrap loader for Cassette
;----------------------------------------------------------------------------
adr8kCas32	equ	0x0000		;load address in 8080

inj8kCas32	dw	low(adr8kCas32)	;8080 load address
	dw	high(adr8kCas32)
	dw	end8kCas32-str8kCas32	;length of program

str8kCas32	equ	$

 dw 0x21,0xAE,0x1F,0x31,0x12,0x00,0xDB,0x06,0x0F,0xD8,0xDB,0x07,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

end8kCas32	dw	low(adr8kCas32)	;8080 start address
	dw	high(adr8kCas32)

;----------------------------------------------------------------------------
; 4K BASIC v4.0 bootstrap loader for 88-2SIO
;----------------------------------------------------------------------------
adr4k2sio40	equ	0x0000		;load address in 8080

inj4k2sio40	dw	low(adr4k2sio40)	;8080 load address
	dw	high(adr4k2sio40)
	dw	end4k2sio40-str4k2sio40	;length of program

str4k2sio40	equ	$

 dw 0x3E,0x03,0xD3,0x10,0x3E,0x15,0xD3,0x10,0x21,0xC2,0x0F,0x31,0x1A,0x00,0xDB,0x10
 dw 0x0F,0xD0,0xDB,0x11,0xBD,0xC8,0x2D,0x77,0xC0,0xE9,0x0B,0x00

end4k2sio40	dw	low(adr4k2sio40)	;8080 start address
	dw	high(adr4k2sio40)

;----------------------------------------------------------------------------
; 4K BASIC v4.0 bootstrap loader for 88-SIO
;----------------------------------------------------------------------------
adr4kSio40	equ	0x0000		;load address in 8080

inj4kSio40	dw	low(adr4kSio40)	;8080 load address
	dw	high(adr4kSio40)
	dw	end4kSio40-str4kSio40	;length of program

str4kSio40	equ	$

 dw 0x21,0xC2,0x0F,0x31,0x12,0x00,0xDB,0x00,0x0F,0xD8,0xDB,0x01,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

end4kSio40	dw	low(adr4kSio40)	;8080 start address
	dw	high(adr4kSio40)


;----------------------------------------------------------------------------
; 4K BASIC v4.0 bootstrap loader for Cassette
;----------------------------------------------------------------------------
adr4kCas40	equ	0x0000		;load address in 8080

inj4kCas40	dw	low(adr4kCas40)	;8080 load address
	dw	high(adr4kCas40)
	dw	end4kCas40-str4kCas40	;length of program

str4kCas40	equ	$

 dw 0x21,0xC2,0x0F,0x31,0x12,0x00,0xDB,0x06,0x0F,0xD8,0xDB,0x07,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

end4kCas40	dw	low(adr4kCas40)	;8080 start address
	dw	high(adr4kCas40)

;----------------------------------------------------------------------------
; 8K BASIC v4.0 bootstrap loader for 88-2SIO
;----------------------------------------------------------------------------
adr8k2sio40	equ	0x0000		;load address in 8080

inj8k2sio40	dw	low(adr8k2sio40)	;8080 load address
	dw	high(adr8k2sio40)
	dw	end8k2sio40-str8k2sio40	;length of program

str8k2sio40	equ	$

 dw 0x3E,0x03,0xD3,0x10,0x3E,0x15,0xD3,0x10,0x21,0xC2,0x1F,0x31,0x1A,0x00,0xDB,0x10
 dw 0x0F,0xD0,0xDB,0x11,0xBD,0xC8,0x2D,0x77,0xC0,0xE9,0x0B,0x00

end8k2sio40	dw	low(adr8k2sio40)	;8080 start address
	dw	high(adr8k2sio40)

;----------------------------------------------------------------------------
; 8K BASIC v4.0 bootstrap loader for 88-SIO
;----------------------------------------------------------------------------
adr8kSio40	equ	0x0000		;load address in 8080

inj8kSio40	dw	low(adr8kSio40)	;8080 load address
	dw	high(adr8kSio40)
	dw	end8kSio40-str8kSio40	;length of program

str8kSio40	equ	$

 dw 0x21,0xC2,0x1F,0x31,0x12,0x00,0xDB,0x00,0x0F,0xD8,0xDB,0x01,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

end8kSio40	dw	low(adr8kSio40)	;8080 start address
	dw	high(adr8kSio40)

;----------------------------------------------------------------------------
; 8K BASIC v4.0 bootstrap loader for Cassette
;----------------------------------------------------------------------------
adr8kCas40	equ	0x0000		;load address in 8080

inj8kCas40	dw	low(adr8kCas40)	;8080 load address
	dw	high(adr8kCas40)
	dw	end8kCas40-str8kCas40	;length of program

str8kCas40	equ	$

 dw 0x21,0xC2,0x1F,0x31,0x12,0x00,0xDB,0x06,0x0F,0xD8,0xDB,0x07,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

end8kCas40	dw	low(adr8kCas40)	;8080 start address
	dw	high(adr8kCas40)

;----------------------------------------------------------------------------
; Extended BASIC v4.0 bootstrap loader for 88-2SIO
;----------------------------------------------------------------------------
adrEx2sio40	equ	0x0000		;load address in 8080

injEx2sio40	dw	low(adrEx2sio40)	;8080 load address
	dw	high(adrEx2sio40)
	dw	endEx2sio40-strEx2sio40	;length of program

strEx2sio40	equ	$

 dw 0x3E,0x03,0xD3,0x10,0x3E,0x15,0xD3,0x10,0x21,0xC2,0x3F,0x31,0x1A,0x00,0xDB,0x10
 dw 0x0F,0xD0,0xDB,0x11,0xBD,0xC8,0x2D,0x77,0xC0,0xE9,0x0B,0x00

endEx2sio40	dw	low(adrEx2sio40)	;8080 start address
	dw	high(adrEx2sio40)

;----------------------------------------------------------------------------
; Extended BASIC v4.0 bootstrap loader for 88-SIO
;----------------------------------------------------------------------------
adrExtSio40	equ	0x0000		;load address in 8080

injExtSio40	dw	low(adrExtSio40)	;8080 load address
	dw	high(adrExtSio40)
	dw	endExtSio40-strExtSio40	;length of program

strExtSio40	equ	$

 dw 0x21,0xC2,0x3F,0x31,0x12,0x00,0xDB,0x00,0x0F,0xD8,0xDB,0x01,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

endExtSio40	dw	low(adrExtSio40)	;8080 start address
	dw	high(adrExtSio40)

;----------------------------------------------------------------------------
; Extended BASIC v4.0 bootstrap loader for Cassette
;----------------------------------------------------------------------------
adrExtCas40	equ	0x0000		;load address in 8080

injExtCas40	dw	low(adrExtCas40)	;8080 load address
	dw	high(adrExtCas40)
	dw	endExtCas40-strExtCas40	;length of program

strExtCas40	equ	$

 dw 0x21,0xC2,0x3F,0x31,0x12,0x00,0xDB,0x06,0x0F,0xD8,0xDB,0x07,0xBD,0xC8,0x2D,0x77
 dw 0xC0,0xE9,0x03,0x00

endExtCas40	dw	low(adrExtCas40)	;8080 start address
	dw	high(adrExtCas40)

;----------------------------------------------------------------------------
; v3.2 Cassette Leader Detector
;----------------------------------------------------------------------------
adrLdrDet32	equ	0x0100		;load address in 8080

injLdrDet32	dw	low(adrLdrDet32)	;8080 load address
	dw	high(adrLdrDet32)
	dw	endLdrDet32-strLdrDet32	;length of program

strLdrDet32	equ	$

 dw 0xDB,0x07,0xFE,0xAE,0xC2,0x00,0x01,0xC3,0x00,0x00

endLdrDet32	dw	low(adrLdrDet32)	;8080 start address
	dw	high(adrLdrDet32)

;----------------------------------------------------------------------------
; v4.0 Cassette Leader Detector
;----------------------------------------------------------------------------
adrLdrDet40	equ	0x0100		;load address in 8080

injLdrDet40	dw	low(adrLdrDet40)	;8080 load address
	dw	high(adrLdrDet40)
	dw	endLdrDet40-strLdrDet40	;length of program

strLdrDet40	equ	$

 dw 0xDB,0x07,0xFE,0xC2,0xC2,0x00,0x01,0xC3,0x00,0x00

endLdrDet40	dw	low(adrLdrDet40)	;8080 start address
	dw	high(adrLdrDet40)

;----------------------------------------------------------------------------
; 88-2SIO Echo (1st port)
;----------------------------------------------------------------------------
adrEco2sioA	equ	0x0000		;load address in 8080

injEco2sioA	dw	low(adrEco2sioA)	;8080 load address
	dw	high(adrEco2sioA)
	dw	endEco2sioA-strEco2sioA	;length of program

strEco2sioA	equ	$

 dw 0x3E,0x03,0xD3,0x10,0x3E,0x15,0xD3,0x10,0xDB,0x10,0x0F,0xD2,0x08,0x00,0xDB,0x11
 dw 0xD3,0x11,0xC3,0x08,0x00

endEco2sioA	dw	low(adrEco2sioA)	;8080 start address
	dw	high(adrEco2sioA)

;----------------------------------------------------------------------------
; 88-2SIO Echo (2nd port)
;----------------------------------------------------------------------------
adrEco2sioB	equ	0x0000		;load address in 8080

injEco2sioB	dw	low(adrEco2sioB)	;8080 load address
	dw	high(adrEco2sioB)
	dw	endEco2sioB-strEco2sioB	;length of program

strEco2sioB	equ	$

 dw 0x3E,0x03,0xD3,0x12,0x3E,0x15,0xD3,0x12,0xDB,0x12,0x0F,0xD2,0x08,0x00,0xDB,0x13
 dw 0xD3,0x13,0xC3,0x08,0x00

endEco2sioB	dw	low(adrEco2sioB)	;8080 start address
	dw	high(adrEco2sioB)

;----------------------------------------------------------------------------
; 88-SIO Echo
;----------------------------------------------------------------------------
adrEcoSio	equ	0x0000		;load address in 8080

injEcoSio	dw	low(adrEcoSio)	;8080 load address
	dw	high(adrEcoSio)
	dw	endEcoSio-strEcoSio	;length of program

strEcoSio	equ	$

 dw 0xDB,0x00,0x0F,0xDA,0x00,0x00,0xDB,0x01,0xD3,0x01,0xC3,0x00,0x00

endEcoSio	dw	low(adrEcoSio)	;8080 start address
	dw	high(adrEcoSio)

;----------------------------------------------------------------------------
; Cassette Port Echo
;----------------------------------------------------------------------------
adrEcoCas	equ	0x0000		;load address in 8080

injEcoCas	dw	low(adrEcoCas)	;8080 load address
	dw	high(adrEcoCas)
	dw	endEcoCas-strEcoCas	;length of program

strEcoCas	equ	$

 dw 0xDB,0x06,0x0F,0xDA,0x00,0x00,0xDB,0x07,0xD3,0x07,0xC3,0x00,0x00

endEcoCas	dw	low(adrEcoCas)	;8080 start address
	dw	high(adrEcoCas)

;----------------------------------------------------------------------------
; Kill the Bit
;----------------------------------------------------------------------------
adrKillBit	equ	0x0000		;load address in 8080

injKillBit	dw	low(adrKillBit)	;8080 load address
	dw	high(adrKillBit)
	dw	endKillBit-strKillBit	;length of program
	
strKillBit	equ	$

 dw 0x21,0x00,0x00,0x16,0x80,0x01,0x0E,0x00,0x1A,0x1A,0x1A,0x1A,0x09,0xD2,0x08,0x00
 dw 0xDB,0xFF,0xAA,0x0F,0x57,0xC3,0x08,0x00
	
endKillBit	dw	low(adrKillBit)	;8080 start address
	dw	high(adrKillBit)

;----------------------------------------------------------------------------
; Pong
;----------------------------------------------------------------------------
adrPong	equ	0x0000		;load address in 8080

injPong	dw	low(adrPong)	;8080 load address
	dw	high(adrPong)
	dw	endPong-strPong	;length of program
	
strPong	equ	$

 dw 0x01,0x0E,0x00,0x31,0x7D,0x00,0x21,0x00,0x00,0x22,0x80,0x00,0x11,0x00,0x80,0xC3
 dw 0x1E,0x00,0x21,0x00,0x00,0x1A,0x1A,0x1A,0x1A,0x09,0xD2,0x15,0x00,0xC9,0xCD,0x12
 dw 0x00,0xDB,0xFF,0xE6,0x01,0xCA,0x2D,0x00,0x7A,0xB3,0xE6,0x1F,0x5F,0x7A,0xE6,0x01
 dw 0xCA,0x45,0x00,0x7B,0xE6,0x02,0xC2,0x3F,0x00,0x7B,0xE6,0x01,0xC2,0x73,0x00,0x21
 dw 0x81,0x00,0x34,0x1E,0x00,0x7A,0x0F,0x57,0xC3,0x1E,0x00,0xCD,0x12,0x00,0xDB,0xFF
 dw 0xE6,0x80,0xCA,0x5D,0x00,0x7A,0x0F,0x0F,0x0F,0xB3,0xE6,0x1F,0x5F,0x7A,0xE6,0x80
 dw 0xCA,0x75,0x00,0x7B,0xE6,0x08,0xC2,0x6F,0x00,0x7B,0xE6,0x10,0xC2,0x43,0x00,0x21
 dw 0x80,0x00,0x34,0x1E,0x00,0x7A,0x07,0x57,0xC3,0x4B,0x00

endPong	dw	low(adrPong)	;8080 start address
	dw	high(adrPong)

	end

